;Don't change the following information !

                TITL    "Ev14.043 Copyright (c) Hughes Power Control Systems 1995 - Unpublished Work - All Rights Reserved - HUGHES PROPRIETARY"
                PAGE    76                ;Lines per printed page

;Don't change the above information!

; Ev14.043
;
;  Changed Regulate_Mode1 to use v_filt instead of v_raw.  v_raw was preventing
;  us from reaching our trip point.
;
;  Increased accuracy in calculating amphours .vs. percentage desired to be
;  returned.
;
;  Added AMPHR_TRIP_3.
;
;  Added AMPHR_TRACK_3.
;
;  Added the ability to track amphours applied in Mode 1 & 2 and then return a
;  percentage of the cumulative amphours into Mode 3.  If using this option,
;  normally the amphour tracking for Mode 1 and 2 would be disabled.
;
;  Removed code for Dolphin units.
;
;  Removed code for Aux Battery monitoring.
;
;  11 Jan 96   JWH


; Ev14.003
;
;  Changed AMPHR_TRIP_AT to AMPHR_TRIP_2.
;
;  Changed AMPHR_TRACKING to AMPHR_TRACK_2.
;
;  Changed BLS to BCS in the Oc2_Irq routine when compaing to Mode 254.
;
;  Added BrownOut detection when charge command is forced to 0 (Ctrl_Neg3:).
;
;  Replaced Read_DiscA and Read_DiscC with Read_Ports.
;
;  Reworked PutChar module to eliminate the JSR RESET_WATCHDOG.  It was taking
;  up unnecessary time and wasting EEProm.
;
;  Changed assorted JSR to BSR for EEProm recovery.
;
;  Renamed Special_Flag to Flag.
;
;  Modified the mode parameter table to recover EEProm.
;
;  Moved INIT_DLCS to the beginning of the program.  This recovered EEProm.
;
;  Updated code to correct operation with PA7.
;
;  02 Nov 95  JWH


; Ev14.002
;
;  Added fault checking for minimum temperature.  If one of the temperature
;  sensors drop below the TEMPMIN value (too cold to charge), the program will
;  enter fault mode.
;
;  Reworked Shift_L_Double and Shift_R_Double for EEProm recovery.
;
;  Temperature Compensation routine changed to accomodate EEProm recovery.
;
;  21 Jun 95  JWH


; Ev14.001
;
;  Changes to code for recovery of EEProm.
;
;  06 Jun 95


; Ev14.000
;
;  Change the v_raw in the CONTROL: module to use v_filt.  It was originally
;  v_filt but was changed to v_raw while perfroming changes in Phoenix.  Also
;  changed Regulate_Mode3: from V_raw to V_filt for greater stability while in
;  Mode 3 with multiple levels.
;
;  Added Flag3, Bit 6 as a Mode 3 delay loop required.  Prevents leaving a
;  level in Mode 3 until the LEVEL_DELAY >= mode3_delay.
;
;  Moved multiple modules around, changed various JSR to BSR, and other
;  assorted tweeking to recover EEProm.  Keep running out of EE.
;
;  Added an upper voltage limiter to Modes 1 & 3.  This was required due to the
;  changes made to Mode 3 and optional levels.
;
;  Changed the original Mode 5 to become known as Mode 254, Completion Mode.
;
;  Changed the original Mode 6 to become known as Mode 255, Fault Mode.
;
;  Replaced Mode 3.  Mode 3 now has the ability to be either a constant current
;  mode (as before), or has the ability to step through multiple levels of
;  constant current before passing on to the next Mode.
;
;  Added the ability for the customer to determine which modes to run and in
;  which order.
;
;  Removed CC_N, CV_N, CC_LIM, and CV_LIM from the parameter table and moved
;  them to the Equates.h file as constants POWER_SCALE and OVERFLOW_LIMIT.
;  This allowed the recovery of 16 ram locations.
;
;  Added tracking of time spent in each mode.
;
;  Cleaned up the code in Tof_Irq.
;
;  Added amphour tracking for all four modes.
;
;  Added a "rounding up" value to Vtrip & Vset equations in the Customer.h file
;  to provide better resolution.  Helps decrease resolution problems with the
;  diagnostic software.
;
;  01 Jun 95  JWH


; EV13.100
;
;  Added Special_Flag3, 04h, as a flag to allow bypassing Mode 2 & Mode 4 trip
;  points if the flag is set.
;
;  Added a rate of change limiter when approaching a set point.  This limiter
;  takes over when SetPoint - Measured < 10 lsb's.
;
;  Removed Slew Rate Limiters from the Ctrl: module.
;
;  Removed comments concerning updates to this program prior to EV13.
;
;  Addition of Maximum Amphours (AmpHr_MaxLimit) returned to the battery safety
;  check.  Mode 6 (Fault Mode) is entered if the cumulative amp hours applied
;  during a charging cycle exceeds this value.
;
;  Added an upper current limit to Modes 2 & 4.  This limiter prevents current
;  (amps) runaway from occurring.  This situation could happen with a defective
;  battery, difference in set/trip points between Mode 1 & Mode 2, etc..
;
;  Cleaned up the code for Tof_Irq
;
;  10 May 95  JWH


; EV13.001
;
;  Changed to correspond with the EV13001.exe file.  No code changes.


; EV13.000
;
;  Added 2 analog - digital channels, PE4 & PE5 (AN4 & AN5).  PE4 is used for
;  monitoring the auxiliary battery.  PE5 is used for monitoring the precharge
;  circuitry in the Dolphin unit.
;
;  Corrected bug with Mode 2 charge command driving to maximum.  Restored the
;  v_ctrl2 variable.
;
;  Moved the code which presets Special Flags 1 & 2 to the front end of the
;  program.
;
;  Corrected code for transfering data to RAM for transmission to the pc.  All
;  of the data wasn't being sent.
;
;  Moved scaling factors for Voltage and Current to the Customer.h file.
;
;  Moved I_COR & V_COR gain factors to the customer.h file.
;
;  Moved POS_SLEW_RATE_LIMIT & NEG_SLEW_RATE_LIMIT factors to the customer.h
;  file.
;
;  Corrected logic with Mode 2 AmpHr counting.
;
;  14 Mar 95  JWH


;****************************************************************************
 
                incl    "equates.h"

;****************************************************************************
;  This is where the Ram storage starts.
;
                org     0h                ; Ram data area
;
                incl    "customer.h"      ; Customer Modifiable Variables
                incl    "ramequs.h"       ; Defined Ram Storage Variables

;******************************************************************************
;  Startup calculations
;
;  This constant determines the maximum number of amp mins allowed to be
;  applied to the battery pack before entering fault mode.
;
MAX_AMP_MINS:   equ     AmpHr_MaxLimit*60 ;60 mins per hr * Max Amp Hours

;  This constant defines how many amps per level are used in Mode 5.
;
AMPSPERLEVEL:   equ     {{I2*AMPS_SCALE}/{AMPS_MULTIPLY*Ten}}/Levels

;  This equation loads the 1st and 2nd modes into a common term.
;
FIRST_SECOND:   equ     {Second_Mode*16}+First_Mode

;  This equation loads the 3rd and 4th modes into a common term.
;
THIRD_FOURTH:   equ     {Fourth_Mode*16}+Third_Mode

;****************************************************************************
;  RESET (initialization)
;  Description:  Sets 68HC11E2 registers to known states.   
;  Prog flow:    Code is executed on reset, jumps to main on completion.
;  Regs Altered: a, s, x
;  Global Read:  None.
;  Global Write: None.
;  Passed Value: None.
;  Return value: None.
;
;  Note: On RESET the following HC11 registers are defined as follows
;        Register       Reset State        Comments
;     *  ADCTL          00UU,UUUU          Reset below
;        ADR1-4         XXXX,XXXX          Read Only (Data)
;     *  BAUD           0000,0UUU          SCI Baud Reset below
;     *  BPROT          1110,0000          EEPROM protect reset below
;        CFORC          0000,0000          Timer compare force reg, not used
;     *  CONFIG         0000,0UUU          Implemented in EEPROM
;        COPRST         UUUU,UUUU          COP reset reg, not used
;     *  DDRC           0000,0000          Data dir C, reset below
;     *  DDRD           0000,0000          Data dir D, reset below
;        HPRIO          UUUU,0101          IRQ change. OC1 hi pri (set below)
;        INIT           0000,0000          RAM/IO mapping unchanged
;     *  OC1D           0000,0000          Out comp 1 data set below
;     *  OC1M           0000,0000          Out comp 1 mask set below
;     *  OPTION         0001,0000          Option reset below
;        PACNT          XXXX,XXXX          Pulse accum data not used
;     *  PACTL          0000,0000          Pulse accum reg reset below
;        PIOC           0000,0U11          Parallel i/o control not used
;        PORTA          Z000,0ZZZ          Port A Data
;        PORTB          0000,0000          Port B Data
;        PORTC          ZZZZ,ZZZZ          Port C Data Hi Z inputs
;        PORTCL         ????,????          Port C Latched Data not used
;        PORTD          01ZZ,ZZZZ          Port D Data
;        PORTE          ZZZZ,ZZZZ          Port E Data a/d inputs
;        PPROG          0000,0000          EEPROM Prog reg
;     *  SCCR1          UU00,0000          SCI ctrl reg 1 reset below
;     *  SCCR2          0000,0000          SCI ctrl reg 2 reset below
;        SCDR           XXXX,XXXX          SCI data reg
;        SCSR           1100,0000          SCI status reg
;     *  SPCR           0000,01UU          SPI control reg reset below
;        SPDR           XXXX,XXXX          SPI data reg
;        SPSR           0000,0000          SPI status reg
;        TCNT           0000,0000          Timer reg, 16 bit, 0000 on reset
;     *  TCTL1          0000,0000          Timer ctrl 1, set below
;        TCTL2          0000,0000          Timer ctrl 2, capture disabled
;        TEST1          0000,X000          Factory test, not used
;        TFLG1          0000,0000          Timer out comp/in capt IRQ status
;        TFLG2          0000,0000          Timer TOF, RTI, PW IRQ status
;        TIC1-3         0000,0000          Input capture regs, not used
;     *  TMSK1          0000,0000          Main timer IRQ msk, set below
;     *  TMSK2          0000,0000          Main timer IRQ msk, set below
;        TOC1-5         0000,0000          Timer Out comp regs, not used
;

;****************************************************************************
;  This is where the program starts in the EEPROM
;
RESET:          org     0f800h            ;start at top eeprom

;****************************************************************************
;  Register Initialization
;
                ldx     #REGBAS           ;offset to registers
                ldaa    #93h              ;Enable charge pump, A/D->E
                staa    OPTION,x          ;cop timeout 1.049 sec
                clra                      ;disable eeprom protection
                staa    BPROT,x           ;Clear the Block Protect Register
                staa    SCCR1,x           ;configure sci serial port

;  8 data bits 1 stop bit
;
                ldaa    #00001011b        ;Set OC1 priority IRQ
                staa    HPRIO,x           ;OC1 highest maskable pri

                ldd     #2000             ;Timer Functions
                std     TOC1,x            ;Set 1ms irq cycle for oc1
                                          ; 1msec/.5us/tick = 2000

                ldd     #20000            ;Set 10ms irq cycle for oc2
                std     TOC2,x            ; 10ms/.5us/tick =
 
                ldaa    #11111100b        ;Port C DDR
                staa    DDRC,x            ;C7-C2 Outputs, C1-C0 Inputs
 
                ldaa    #00111010b        ;Port D DDR
                staa    DDRD,x            ;D5,4,3,1 outputs 2,0 inputs
 
                ldaa    #10001000b        ;Pulse Accumulator Control Reg
                staa    PACTL,x           ;DA7, DA3 out, RTI IRQ e/2^16
 
                ldaa    #00001100b        ;enable sci serial port
                staa    SCCR2,x           ;rx, tx activated, no irq
 
                ldaa    #00110000b        ;set baud of 9600
                staa    BAUD,x            ;
 
                ldaa    #01010000b        ;spi control reg
                staa    SPCR,x            ;No IRQ, Sys Enble, clk E/2
 
;****************************************************************************
;  Variable Initialization
 
;  ram wipe : This is where we clear all RAM memory.
;
                ldx     #0                ;Get the starting address
                clra                      ;Clear the Accumulator
ram_wipe:       staa    0,x               ;Clear this address
                inx                       ;Increment our address offset index
                cpx     #0ffh             ;Have we cleared 255 bytes yet
                bls     ram_wipe          ;No, loop till we're done
 
                lds     #0ffh             ;stack at top of ram
                ldaa    #08h              ;Init b0 coefficient
                staa    vb                ;8 bit fraction: 0.03125
                staa    ib                ;8 bit fraction: 0.03125
                staa    t1b               ;8 bit fraction: 0.03125
                staa    t2b               ;8 bit fraction: 0.03125
 
                ldaa    #0f0h             ;Init a1 coefficient
                staa    va                ;8 bit fraction: 0.93750
                staa    ia                ;8 bit fraction: 0.93750
                staa    t1a               ;8 bit fraction: 0.93750
                staa    t2a               ;8 bit fraction: 0.93750
 
                ldaa    #244              ;Equates to 1 second
                staa    four_msec_count   ;Preset power on counter to 1 second
                staa    mfour_msec_count  ;Preset the mode timer to 1 second


;****************************************************************************
;  Init_Dlcs 
;  Description:  Initializes the DLCS.  DLCS is reset, a config/commandbyte
;                is sent to the DLCS and a short delay is implemented to
;                ensure latching of the configuration data.
;  Prog flow:    Linear execution, calls putdlcs.
;  Regs Altered: x, a, b
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;
Init_Dlcs:      ldx     #REGBAS           ;Offset to registers
                bclr    PORTA,x,RST       ;Assert DLCS *RST
                nop                       ;sleep for .5us
                nop                       ;sleep for .5us
                bset    PORTA,x,RST       ;DLCS *RST pin
                                          ;Negate DLCS *RST
                clra                      ;DLCS config: 0000,0000
                                          ;No tst, irq en, 00 osc, 0 4x
                ldab    #18h              ;Load Cmd config:  0001,1000
                                          ;Gen: NOP, Typ: #cnfg, Rx: NOP
                jsr     PutDlcs           ;Send message to DLCS

;****************************************************************************
;  Start the A/D conv.
;
                ldaa    #30               ; initiate the A/D
                staa    ADCTL,x           ; a/d conversion
 
;****************************************************************************
;  Interrupts Enabled
;
                ldaa   #01000000b         ;Timer Functions
                staa   TMSK2,x            ;IRQ on  RTI
                                          ; E/1 = .5us/cycle
 
                bset   TMSK1,x, 80h       ;enable oc1 irq (a/d)
 
                                          ; mode=0 (pud)
                cli                       ;enable rti irq
                                          ;system clock starts ticking
 
;****************************************************************************
;  A/D Settling Time  Delay
;
a2d_wait:       jsr     reset_watchdog    ;give analog 1sec to settle
                ldaa    four_msec_count   ;See how long it's been
                cmpa    #1                ;Have we waited long enough
                bhi     a2d_wait          ;No, keep waiting
 
;****************************************************************************
;  Read the ports, then set PA7 (Discrete Output).  X Register is set in the
;  Read_Ports routine so no need to set it here.
;
set_contact:    jsr     Read_Ports        ;Get state of Port A, and save it
                bset    PORTA,x,080h      ;Turn on PA7, Discrete Output

;****************************************************************************
;  Power Up Delay
;
                ldaa    #pud              ;Get the power up delay
pud_loop:       jsr     reset_watchdog    ;Go reset the watchdog
                cmpa    one_sec_count     ;Have we waited long enough yet?
                bgt     pud_loop          ;Loop till we time out

;****************************************************************************
;  More Interrupts Enabled
;
;  More_irq label added when this routine was moved from prior to the Power Up
;  Delay loop, to after the loop.
;
More_Irq:       bset    TMSK2,x, 80h      ;enable tof irq (Schedule J1850)
                bset    TMSK1,x, 40h      ;enable oc2 irq (check fault)

;****************************************************************************
;  Calculate State Of Charge at Start - 14 Nov 94 by J.W. Hagen, HTSC
;  Description:  Calculates the approximate state of charge for the battery
;                pack prior to entering Mode 1.  Occurs immediately after the
;                power-up delay routine and during the Calc_Soc module.
;  Prog flow:    Linear execution
;  Regs Altered: D,X
;  Global Read:  volt_start
;  Global Write: None.
;  Passed value: None.
;  Return value: D accumulator contains the batter state of charge when the
;                charging process began, prior to Mode 1.
;
Calc_Soc_Start: ldab    v_raw             ;Get the raw voltage
                stab    v_at_start_up     ;Save as the starting voltage
                cmpb    #VMIN             ;Is it at least Vmin
                bhs     Calc_Soc_Star1    ;Yes, we're ok
                ldab    #VMIN             ;Get Vmin

Calc_Soc_Star1: subb    #ZERO_REF         ;Subtract our zero reference
                bcc     Calc_Soc_Star2    ;Not set if we didn't borrow
                negb                      ;Get the 2's compliment
                bset    Flag1,04h         ;Set bit 2 to show as negative number

Calc_Soc_Star2: ldaa    #100              ;To give us an integer percentage
                mul                       ;Apply it
                ldx     #BATT_ENERGY      ;1.2 * #Batteries
                idiv                      ;Yields the startup State of Charge
                xgdx                      ;Move it to the A&B, use the LSByte
                brset   flag1,04h,Calc_Soc_Star3   ;Flag clear, press on
                cpd     #100              ;How does it compare to 100%
                bhs     Calc_Soc_Star4    ;More than or same as 100%
                bra     Calc_Soc_Star5    ;0-99%, looking good

Calc_Soc_Star3: negb                      ;So we can save it as a negative #
                stab    soc_at_start      ;Save the negative value.  Happens
                                          ;when the battery is really low.
                clrb                      ;For our state of charge indication
                bra     Calc_Soc_Done     ;Branch and save

Calc_Soc_Star4: ldab    #99               ;Hard load a 99% value
Calc_Soc_Star5: stab    soc_at_start      ;Save our starting value
Calc_Soc_Done:  stab    ch_state          ;Our % State of Charge is in the B

;****************************************************************************
;  Ram_Initialize - 19 Jun 95 by J.W. Hagen, HTSC
;  Description:  Load the data for sync bytes, program information, and the
;                data from the customer.h file into ram as required.
;  Prog flow:    Linear.
;  Regs Altered: Y,X,A
;  Global Read:  Ram Data
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
;  We are placing the following data in RAM so we may send them out the port
;  and allow the diagnostic program to utilize the data for calculations and
;  displays to the user.
;
Mode_Out:       ldy     #Ram_Setup        ;Starting address of data to load
                ldx     #Sync_Data        ;Starting ram address to save data

Mode_Out1:      ldaa    0,y               ;Load the Ram data, starting at 00h
                staa    0,x               ;Save in parms location
                inx                       ;Increment our byte counter
                iny                       ;Increment the address offset
                cpy     #Ram_Last_Byte    ;Have we sent all the data yet
                blt     Mode_Out1         ;No, loop till we're done

;****************************************************************************
;
;  Set the required flags into Flag1
;
;  The following routine copies the temperature compensation flag.  If set to
;  1, the temperature compensation routine is active.  If set to 0, the routine
;  is inactive.  The flag becomes Bit 0 in the Flag1 byte.
;
                ldaa    #TEMP_COMP_FLAG   ;Get the Temperature Comp Flag

;  Save the new set of flags for Flag 1.
;
                staa    flag1     ;Save as the new value

;****************************************************************************


;****************************************************************************
;
;  Set the required flags into Flag2
;
;  The following routine copies the Temperature Sensor monitoring flag into
;  Flag 2, Bit 1.  If set to 1, the DTemperature Sensors are monitored for
;  over and under temperature limits.  If any of the limits defined in the
;  customer.h file are exceeded, Fault Mode is entered.
;
                ldaa    #MON_TEMP_SENSE   ;Move MON_TEMP_SENSE to Bit 0

;  The following routine copies the Amp Hour tracking flag.  If set to 1, the
;  amp hours applied while in Mode 1 are tracked and used to control an
;  additional trip point for exiting Mode 2.  If set to 0, the routine is
;  inactive.  The flag becomes Bit 0 in the Flag2 byte.
;
                lsla                      ;Move MON_TEMP_SENSE to Bit 1
                oraa    #AMPHR_TRACK_2    ;Move Amp Hours tracking to Bit 0.

;  Save the new set of flags for Special Flag 2.
;
                staa    flag2     ;Save as the new value

;****************************************************************************
;
;  Set the required flags into Flag3
;
;  The following routine copies the Bypass Mode 2 & Mode 4 Trip Point flag into
;  Special Flag 3, Bit 2.  If set to 1, Mode 2 & Mode 4 trip points are ignored
;  and mode duration is controlled by the amount of time set for the respective
;  mode.
;
                ldaa    #BYPASS_TRIPPT    ;Move Bypass_TripPt to Bit 0.
                lsla                      ;Move Bypass_TripPt to Bit 1.
                lsla                      ;Move Bypass_TripPt to Bit 2.

;  Save the new set of flags for Special Flag 3.
;
                staa    flag3     ;Save as the new value

;  Bit 3 is used to determine a fast or slow iteration on the error
;  generated by SetPoint - Measured in the CTRL: module.  Set the flag to start
;  a fast iteration.
;
                bset    Flag3,49h         ;Set Bit 6 to force Mode 3 Delay Loop
                                          ;Set Bit 3 for Pos Error Calculations
                                          ;Set Bit 0 as Charge Port 1 CP

;****************************************************************************
;
;  Set the required flags into Flag4
;
;  The following routine copies the Amp Hour tracking flag.  If set to 1, the
;  amp hours applied while in Mode 1 are tracked and used to control an
;  additional trip point for exiting Mode 2.  If set to 0, the routine is
;  inactive.  The flag becomes Bit 0 in the Flag2 byte.
;
                ldaa    #AMPHR_TRACK_3    ;Get AmpHr Tracking Modes 1&2 Flag

;  Save the new set of flags for Flag 4.
;
                staa    flag4     ;Save as the new value

;****************************************************************************
;  The sample_periods is used in the CTRL: module to limit the update rate to
;  the charge command.  Preset the counter to a value to force it into an
;  immediate update at startup.
;
                ldaa    #200              ;Set to a high value ( > 100 )
                staa    sample_periods    ;Control Loop sample periods complete

;****************************************************************************
;  Perform last minute changes prior to entering the main program loop.
;
                clr     brownout_secs     ;Reset the counter
                clr     brownout_wait     ;Reset the counter

;****************************************************************************
;  Set up the first mode
;
                jsr     Select_Mode       ;Change to the 1st Mode
 
;****************************************************************************
;  main program
;  Description:  Executes all non-IRQ system and diagnostic functions.
;  Prog flow:    Continuous loop.  All system inputs are handled on interrupt.
;  Regs Altered: A,X   
;  Global Read:  i_raw
;  Global Write: None.
;  return value: None.
;
main:           jsr     temp_comp         ;Perform temperature compensation
                jsr     calc_soc          ;Calculate the State Of Charge
                jsr     ram_out           ;Send RAM data out the RS232 port
                jsr     Clear_PA7         ;Determine status for discrete output
                jsr     Read_Ports        ;Update from Port A & Port C
                bra     main              ;Loop till we quit 

;****************************************************************************


;****************************************************************************
;  tof_irq      Modified 15 May 95  JWH
;
;  Description:  Timer overflow IRQ routine.  Schedules the iteration for
;                processing the main CONTROL: module (determines power
;                settings) and J1850 messages to go out.  The schedule rate
;                is as follows:
;
;                    Irq0 - Determine power setting every 98.3msec (Ch_Command)
;                    Irq1 - Ch_Status message every 131ms (Modes 254&255 only).
;                    Irq2 - Ch_Command message transmits every 196ms
;                    Irq3 - Ch_State message transmits every 1.048sec
;
;  Prog flow:    IRQ service.  Scheduled to happen every 32.77ms.  JWH
;  Regs Altered: x, a.
;  Global Read:  mode
;  Global Write: None.
;  Return value: None.
;
TOF_CLEAR:      equ     7fh               ;bclr to TFLG2 to clear IRQ

;  No J1850 messages are scheduled to occur if in Mode 0.
;
Tof_Irq:        tst     mode              ;if mode=0, then no J1850 messages
                beq     Tof_End           ;This is Mode 0, exit
 
;  Increment our Tof_Irq execution counter.
;
                inc     tof_cnt           ;Keep count of TOF IRQs

;  Schedules iteration of SCM power setting (Charge Command) control loops.
;  The value used to AND with tof_cnt determines the rate in which the CTRL:
;  and CONTROL: modules are processed.  Currently set for 01fh.  This provides
;  an execution rate of 5.085hz = (32.77ms * 6) = every 196msec.
;
Tof_Irq0:       ldaa    tof_cnt           ;Execution cycle counter
                anda    #5h               ;Determine Ch_Command 6 * 32.77msec
                bne     Tof_Irq1          ;Not time for CONTROL: or CTRL:

;  Process the CONTROL: and CTRL: modules to determine the required power
;  setting to be used for the Charge Command.
;
                jsr     Control           ;Review inputs, process outputs
;               bra     Tof_End           ;Exit

;  Determine the present Mode.  If Modes 1-4, send Ch_Command or Ch_State msgs
;  as scheduled.  If Mode 254 or 255, send Charge Status Message only.
;
Tof_Irq1:       ldaa    mode              ;Get the current mode
                cmpa    #MODE254          ;Are we in Completion or Fault Mode
                bcs     Tof_Irq2          ;Not Completion or Fault Mode
                clr     mode_delay        ;Clear the Mode Delay seconds counter
                clr     mode3_delay       ;reset mode3 delay clock

;  Schedules Charge Status Message to J1850 bus, when in Modes 254 or 255 only.
;  The value used to AND with tof_cnt determines the rate in which the Charge
;  Status messages are sent to the J1850 bus.  Currently set for 03h.  This
;  provides an execution rate of 7.63hz = (32.77ms * 4) = every 131msec.
;
                ldaa    tof_cnt           ;Load the execution cycle counter
                anda    #3h               ;Send Ch_Status every 4 * 32.77ms
                cmpa    #1h               ;Time to send ch_status?
                bne     Tof_End           ;Not time to send message, press on

;  Send the Charge Status Message and exit.
;
                bsr     Send_Msg2         ;Send Charge Status Message
                bra     Tof_End           ;Exit

;  Schedules Charge Command Message to J1850 bus, Modes 1 thru 4 only.
;  The value used to AND with tof_cnt determines the rate in which the Charge
;  Command messages are sent to the J1850 bus.  Currently set for 03h.  This
;  provides an execution rate of 7.63hz = (32.77ms * 4) = every 131msec.
;
Tof_Irq2:       ldaa    tof_cnt           ;Load the execution cycle counter
                anda    #3h               ;Send Ch_Command every 4 * 32.77ms
                cmpa    #1h               ;Time to send ch_command?
                bne     Tof_Irq3          ;Not time yet, check Ch_State Message

;  Send the Charge Command Message and exit.
;
                bsr     Send_Msg1         ;Send Charge Command Message
                bra     Tof_End           ;Exit
 
;  Schedules Charge State Message to J1850 bus, Modes 1 thru 4 only.
;  The value used to AND with tof_cnt determines the rate in which the Charge
;  State messages are sent to the J1850 bus.  Currently set for 01h.  This
;  provides an execution rate of .954hz = (32.77ms * 32) = every 1.048sec.
;
Tof_Irq3:       ldaa    tof_cnt           ;Load the execution cycle counter
                anda    #1fh              ;Send Ch_State every 32 * 32.77ms
                cmpa    #1fh              ;Time to send ch_state?
                bne     Tof_End           ;Not time yet, exit

;  Send the Charge State Message and exit.
;
                bsr     Send_Msg3         ;Send Charge State Message

;  This routine sets us up to exit the interrupt.
;
Tof_End:        ldx     #REGBAS           ;Offset to registers
                bclr    TFLG2,x,TOF_CLEAR ;Clear the interrupt
                rti                       ;Return from the interrupt
 
;****************************************************************************


;****************************************************************************
;  send_msg_x (x = 1, 2, 3)
;  Description:  Sends one of three J1850 charging messages.
;  1. Charge Command Message: Subaddress = 0x10, Data: = see ch_command      
;  2. Charge Status Message:  Subaddress = 0x11, Data: = see ch_status
;  3. State of Chrge Message: Subaddress = 0x12, Data: = see ch_state
;  Prog flow:    Linear, called by tof_irq.
;  Regs Altered: a, b, x
;  Global Read:  ch_command, ch_status, ch_state
;  Global Write: None.
;  Return value: None.
;
                                          ;Message #10
c_cmd:          dfb     20h,04h           ;CMD:0000,0100 DAT:0001,0000
                                          ; cmd-a tx byte, dat:address

                                          ;Message #11
c_sts:          dfb     21h,04h           ;CMD:0000,0100 DAT:0001,0001
                                          ; cmd-a tx byte, dat:address

                                          ;Message #12
c_ste:          dfb     22h,04h           ;CMD:0000,0100 DAT:0001,0001
                                          ; cmd-a tx byte, dat:address
 
send_msg1:      ldx     c_cmd             ;Get the adrs info
                ldaa    ch_command        ;Load THE DATA
                bra     send_a_message    ;Send the message
 
send_msg2:      ldx     c_sts             ;Get the adrs info
                ldaa    ch_status         ;Load THE DATA
                bra     send_a_message    ;Send the message
 
send_msg3:      ldx     c_ste             ;Get the adrs info
                ldaa    ch_state          ;Load THE DATA
 
send_a_message: psha                      ;Store data
                bsr     tx_the_header     ;Send the sys header
                xgdx                      ;Get data/cmd for address
                bsr     putdlcs           ;Send the adrs info
                pula                      ;Get data
                ldab    #0ch              ;Load cmd for last data
                bsr     putdlcs           ;Send the cmd/data pair
                rts

;****************************************************************************

 
;****************************************************************************
;  tx_the_header
;  Description:  Loads and sends the standard system header to J1850
;  Prog flow:    Linear, called by send_msgx.
;  Regs Altered: a, b
;  Global Read:  None.
;  Global Write: None.
;  Return value: None.
;
 
tx_the_header:  ldd     #8814h            ;Load data/cmd
                                          ;cmd=14=load as 1st byte of message
                bsr     putdlcs           ;Send the data/cmd pair
 
                ldd     #0f204h           ;Load data/cmd
                                          ;cmd=04=load as trasmit data
                bsr     putdlcs           ;Send the data/cmd pair
 
                ldd     #7204h            ;Load data/cmd
                                          ;cmd=04=load as trasmit data
                bsr     putdlcs           ;Send the data/cmd pair
                rts                       ;Go away

;****************************************************************************


;****************************************************************************
;  putdlcs
;  Description:  Sends command/data seq to the DLCS chip.  Checks for J1850
;                errors in the status byte (bus short).
;  Prog flow:    Linear execution, waits for data transfer complete after
;                each byte sent (data and command).
;  Regs Altered: a, b
;  Global Read:  None.
;  Global Write: bad_status.
;  Passed value: data in reg a, command in reg b.
;  Return value: data in reg b, status in reg a.
;
CS:             equ     010h              ;DLCS *CS bit mask
 
putdlcs:        pshx                      ;Save x
                ldx     #REGBAS           ;Offset to registers
                bclr    PORTA,x,CS        ;Assert DLCS *CS
                staa    SPDR,x            ;Send data byte
                clra                      ;Get set for compare
putdlcs1:       cmpa    SPSR,x            ;SPI transfer ok check
                bpl     putdlcs1          ;Loop
;                brclr   SPDR,x,08h,putdlcs2 ;Check status for short
;                ldaa    SPDR,x            ;Retrieve status
;                staa    bad_status        ;Store bad status
putdlcs2:       ldaa    SPDR,x            ;Retrieve status
                stab    SPDR,x            ;Send commandbyte
                clrb                      ;Get set for compare
putdlcs3:       cmpb    SPSR,x            ;SPI transfer ok check
                bpl     putdlcs3          ;Loop
                ldab    SPDR,x            ;Retrieve recieve data
                bset    PORTA,x,CS        ;Negate DLCS *CS
                pulx                      ;Restore x
                rts

;****************************************************************************


;****************************************************************************
;  rti_irq       Modified 19 Apr 95 by JWH for New modes.
;
;  Description:  Real time IRQ routine. Updates system clocks
;  Prog flow:    IRQ service.  Scheduled to happen every 4.10ms.
;  Regs Altered: x, a, b
;  Global Read:
;  Global Write: None.
;  Return value: None.
;
RTI_CLEAR:      equ     40h               ;Write to TFLG2 to clear IRQ
;
; Four_msec_count pre-loaded with 244, which is 1 seconds worth
; One_sec_count cleared to zero.
;
Rti_Irq:        brclr   Flag1,010h,Rti_Irq2  ;Set indicates mode changed

                bclr    Flag1,010h        ;Clear the Mode Transition flag

;  Since Modes were changed, reset the mode clock timers.
; 
Rti_Irq1:       clr     mone_hour_count   ;reset mode clock
                clr     mone_min_count    ;reset mode clock
                clr     mode_delay        ;reset mode delay clock
                clr     mode3_delay       ;reset mode3 delay clock
                clr     mone_sec_count    ;Clear the seconds counter
                ldaa    #244              ;1 seconds worth of data
                staa    mfour_msec_count  ;reset mode clock

;  This routine tracks the time elapsed since power-up
;  
rti_irq2:       dec     four_msec_count   ;decrement our msecond timer
                bne     rti_irq3          ;No, keep working
                ldaa    #244              ;We completed 1 second, load it again
                staa    four_msec_count   ;re-load 4.1ms counter with 1 second
                inc     BrownOut_Secs     ;Increment BrownOut Seconds Counter
                inc     BrownOut_Wait     ;Increment BrownOut Recovery Counter
                inc     one_sec_count     ;Increment 1 second from the count
                inc     mode_delay        ;Increment 1 sec delay for Mode 2&4
                inc     mode3_delay       ;Increment 1 sec delay for Mode 3
                ldaa    one_sec_count     ;Get the current seconds counter
                cmpa    #60               ;Have we reached 60 seconds yet
                blt     rti_irq3          ;No, keep working
                clr     one_sec_count     ;Clear the seconds counter
                inc     one_min_count     ;Increment the minute counter
                ldaa    one_min_count     ;Let's see what it has
                cmpa    #60               ;Have we reached an hour yet
                blt     rti_irq3          ;Less than an hour, keep working
                clr     one_min_count     ;Clear the minute counter
                inc     one_hour_count    ;Increment the hour counter
 
;  This routine tracks the time elapsed in current mode
;
rti_irq3:       dec     mfour_msec_count  ;Decrement the mode msecond counter
                tst     mfour_msec_count  ;Have we completed 1 second yet
                bne     rti_end           ;No, bail out
                ldaa    #244              ;Yes, load equal to 1 second
                staa    mfour_msec_count  ;re-load 4.1ms counter with 1 second
                inc     mone_sec_count    ;Increment the seconds counter
                ldaa    mone_sec_count    ;Get the current mode seconds counter
                cmpa    #60               ;Have we reached 60 seconds yet
                blt     rti_end           ;No, bail out
                clr     mone_sec_count    ;Clear the mode seconds counter
                inc     mone_min_count    ;Increment the minute counter
                ldaa    mone_min_count    ;Let's see what's there
                suba    #60               ;Have we completed an hour yet
                blt     rti_end           ;No, press on
                clr     mone_min_count    ;Clear the mode minute counter
                inc     mone_hour_count   ;Increment the mode hour counter

;  Update the elapsed timer within the mode parameter table, if applicable.
;
Rti_End:        jsr     Updt_Mode_Time    ;Update time within parameter table
                ldx     #REGBAS           ;Load the base address
                ldaa    #RTI_CLEAR        ;Load RTI clear mask
                staa    TFLG2,x           ;Clear RTI
                rti                       ;Return from the interrupt
 
;****************************************************************************

 
;****************************************************************************
;  oc1_irq
;  Description:  Retrieves v, i, temp a/d values and calls iir filter routine
;                scheduled to happen every 1msec
;
;                 Modified to select Group 1 (Channels PE0-PE3) on the odd
;                number pass; set a flag, then select Group 2 (Channels
;                PE4-PE7) on the even number pass.  In reality, each A/D is
;                being read every 2msec.  11 Mar 95  JWH
;
;  Prog flow:    IRQ service.  Calls iir_v iir_t
;  Regs Altered: x, a, b.
;  Global Read:  v_raw, i_raw, t1_raw, t2_raw
;  Global Write: v_raw, v_filt, t1_raw, t1_filt, t2_raw, t2_filt
;  Return value: None.
;
OC1_CLEAR:      equ     07fh              ;bclr to TFLG1 to clear IRQ
V_OFFSET:       equ     0                 ;Voltage Offset Value
I_OFFSET:       equ     0                 ;Current Offset Value
TEMP1_OFFSET:   equ     0                 ;temp1 offset value
TEMP2_OFFSET:   equ     0                 ;temp2 offset value

Oc1_Irq:        ldx     #REGBAS           ;Offset to registers
                ldd     TOC1,x            ;Get old compare value
                addd    #2000             ;Set 1ms irq cycle for oc1
                std     TOC1,x            ;Set 1ms irq cycle for oc1
                                          ; 1msec/.5us/tick = 2000

Oc1_Group_1:    brset   Flag2,80h,Oc1_Group_2   ;Flag set, do next group

;  Read analog channels PE0 - PE3, then set a flag indicating these channels
;  have been read.
;
                ldaa    ADR1,x            ;get v a/d reading
                staa    v_raw             ;store to raw loc

                ldab    ADR2,x            ;get i a/d reading
                stab    i_raw             ;store to raw loc

                ldaa    ADR3,x            ;read temp1
                staa    t1_raw            ;save temp1 reading
 
                ldab    ADR4,x            ;read temp2
                stab    t2_raw            ;save temp2 reading
 
;  Process Raw Voltage and filter it
;
                ldaa    v_raw             ;get input to filter
                staa    vx                ;store to filter input
                ldx     #vx               ;regX=address of vx
                bsr     iir               ;do iir
                std     v_filt            ;Store result

;  Process Raw Current and filter it
;
                ldaa    i_raw             ;get input to filter
                staa    ix                ;store to filter input
                ldx     #ix               ;regX=address of vx
                bsr     iir               ;do iir
                std     i_filt            ;Store result
 
;  Process Raw Temperature Sensor 1 and filter it
;
                ldaa    t1_raw            ;get input to filter
                staa    t1x               ;store to filter input
                ldx     #t1x              ;regX=address of t1x
                bsr     iir               ;do iir
                std     t1_filt           ;Store result
 
;  Process Raw Temperature Sensor 1 and filter it
;
                ldaa    t2_raw            ;get input to filter
                staa    t2x               ;store to filter input
                ldx     #t2x              ;regX=address of t2x
                bsr     iir               ;do iir
                std     t2_filt           ;Store result

                ldx     #REGBAS           ;offset to registers
                ldaa    #34h              ;Set for ch 4-7 a/d once
                staa    ADCTL,x           ;Start. Use 'em next time

                bset    Flag2,80h         ;Set to indicate 1st group is done
                bra     Oc1_Irq_Exit      ;Finished with Group 1, exit

;  Read analog channels PE4 - PE5, then clear a flag indicating these channels
;  have been read.
;
Oc1_Group_2:    ldaa    #10h              ;Set for ch 0-3 a/d once
                staa    ADCTL,x           ;Start. Use 'em next time
                bclr    Flag2,80h         ;Clear to indicate 2nd group is done

;  This routine sets us up to exit the interrupt.
;
Oc1_Irq_Exit:   ldx     #REGBAS           ;Offset to registers
                bclr    TFLG1,x,OC1_CLEAR ;Clear IRQ
                rti                       ;Return from the interrupt

;****************************************************************************
 

;****************************************************************************
;  iir
;  Description:  Infinite Impulse Response filter for temp & voltage
;                measurements.
;  Prog flow:    Called from oc1_irq, linear execution.
;  Regs Altered: x, a, b.
;  Global Read:  16 bit result in location whose address is regX + 0
;  Global Write: None.
;  Return value: 16 bit result in regD
;
B:              equ     0
BX:             equ     2
W:              equ     4
A:              equ     7
Y:              equ     8

iir:            ldd     B,x               ;Get x and b in d
                mul                       ;x * b -> d
                std     BX,x              ;x * b -> bx
                addd    W,x               ;x * b + high 16 bytes of w
                std     Y,x               ;Store result
                clr     W,x               ;Clear high byte of w
                ldaa    A,x               ;Get a coefficient
                ldab    Y+1,x             ;Get low byte of y
                mul                       ;a * y lo -> d
                std     W+1,x             ;a * y lo -> 1st 16 lsbs of w
                ldd     A,x               ;Load a and y in d
                mul                       ;a * y hi -> d
                addd    W,x               ;a * y hi -> 1st 16 msbs of w
                addd    BX,x              ;x * b + a * y  -> d
                std     W,x               ;x * b + a * y  -> d
                ldd     Y,x
                ldx     #REGBAS           ;restore register offset to regX
                rts                       ;done
 
;****************************************************************************


;******************************************************************************
;  oc2_irq       Modified 13 Sep 95  JWH
;
;  Description:  Check for system faults (Mode 255) by looking for any or all
;                of the charge status conditions listed below.  Do not process
;                this routine if already in Completion Mode or Fault Mode.
;  Prog flow:    IRQ service
;  Regs Altered: y, a, b, x
;  Global Read:  
;  Global Write: ch_status.
;  Passed value: None.
;  Return value: Status in reg A.
;
;  NOTE:  In the following bit patterns, two bits have specific meaning.  They
;         are as follows:
;
;           Bit 0 indicates Charge Complete.
;           Bit 1 indicates Charging Failure
;
;     01h   =  Charge is Complete                                 0000 0001
;     06h   =  V_Filt  < VMIN                                     0000 0110
;     0Ah   =  T1_Filt > TempMax                                  0000 1010
;     0Eh   =  Contactor (Discrete PA2) Failed To Engage/Hold     0000 1110

;     12h   =  T2_Filt > TempMax                                  0001 0010
;     16h   =  SCM J1850 Transmitter Failed (BrownOut)            0001 0110
;     1Ah   =  Auxiliary Battery Under Voltage                    0001 1010
;     1Eh   =  Auxiliary Battery Over Voltage                     0001 1110

;     22h   =  One_Hour_Count > TMax                              0010 0010
;     26h   =  Dolphin Precharge FET is OFF                       0010 0110
;     2Ah   =  Dolphin Lid Switch if Open                         0010 1010
;     2Eh   =  Dolphin Fet Error, Unknown as to why.              0010 1110

;     42h   =  Temp Discrete Failure, Port A                      0100 0010
;     46h   =  Tot_Amp_Mins > Max_Amp_Mins (Exceeded Amphr Limit) 0100 0110
;     4Ah   =  Charging Time Limit Exceeded - Hours               0100 1010
;     4Eh   =  Charging Time Limit Exceeded - Hours & Minutes     0100 1110

;     82h   =  T1_Filt < TempMin                                  1000 0010
;     86h   =  V_Filt > VMAX                                      1000 0110
;     8Ah   =  T2_Filt < TempMin                                  1000 1010
;     8Eh   =  Conversion Box Overtemp                            1000 1110

;     92h   =  Conversion Box Thermal Switch is Open              1001 0010
;     96h   =  Charge Port RTD Device Shorted                     1001 0110
;     9Ah   =  Charge Port Thermistor Open                        1001 1010
;     9Eh   =  Spare                                              1001 1110

;     A2h   =  Spare                                              1010 0010
;     A6h   =  Spare                                              1010 0110
;     AAh   =  Spare                                              1010 1010
;     AEh   =  Spare                                              1010 1110

;     B2h   =  Spare                                              1011 0010
;     B6h   =  Spare                                              1011 0110
;     BAh   =  Spare                                              1011 1010
;     BEh   =  Spare                                              1011 1110

;     C2h   =  Temp Discrete Failure, Port C                      1100 0010
;     C6h   =  Spare                                              1100 0110
;     CAh   =  Spare                                              1100 1010
;     CEh   =  Spare                                              1100 1110

;     D2h   =  Badicheq Communications Lost                       1101 0010
;     D6h   =  Badicheq Sending Incorrect Frequency Pulses        1101 0110
;     DAh   =  Badicheq Charge Command > 255 exceeded 5 times     1101 1010
;     DEh   =  Spare                                              1101 1110

;
OC2_CLEAR:      equ     0bfh              ;bclr to TFLG1 to clear IRQ

;  Find out the present Mode.  If Completion or Fault Mode, don't process any
;  thing else here.  If in Fault Mode, one error was already found.  If 
;  Completion Mode, don't need to watch for errors since charging is complete.
;
Oc2_Irq:        ldaa    mode              ;What is the present mode
                cmpa    #MODE254          ;Is it Completion or Fault Mode
                bcs     Chk_Vmax          ;No, process safety routine
                jmp     Chk_Fault_End     ;Finished here

;  Check vehicle Voltage Sense (Vsense) against Voltage Maximum to see if we're
;  doing okay.  If not, set up as a fault.  Modified 07Jun94  JWH
;
;  check for Vsense > Vmax
;
Chk_Vmax:       ldaa    v_filt            ;Get the filtered voltage
                cmpa    #VMAX             ;Is it too much
                bls     Chk_Vmin          ;No, looks good, check Vmin
                bset    Ch_Status, 86h    ;update status word
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check for Voltage Sense (Vsense) against Voltage Minimum to see if we're
;  doing okay.  If not, set up as a fault.
;
;  check for Vsense < Vmin fault
;
Chk_Vmin:       ldaa    v_filt            ;Get the filtered voltage
                cmpa    #VMIN             ;Are we below minimums
                bhs     Chk_Temp_Sense    ;No, let's check the temperature
                bset    Ch_Status, 06h    ;update status word
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check to see if temperature sensor monitoring is enabled.  If not, bypass
;  the checks.
;
Chk_Temp_Sense: brclr   Flag2,02h,Chk_ToFault   ;Flag off, skip checks

;  Check for Temperature sense 1 (Tsens1) against Temperature Maximum to see
;  if we've exceeded TEMPMAX.  If so, set up as a fault.
;
;  Tsens1 > TEMPMAX is our comparison
;
Chk_T1_Max:     ldaa    t1_filt           ;Get the filtered temperature
                cmpa    #TEMPMAX          ;Have we reached the max temp
                bcs     Chk_T1_Min        ;No, look at the min
                bset    ch_status, 0ah    ;Update status word
Set_Fault_Mode: bra     Set_FaultMode     ;Now in fault mode

;  Check for Temperature sense 1 (Tsens1) against Temperature Minimum to see
;  if we've fallen below TEMPMIN.  If so, set up as a fault.
;
;  Tsens1 < TEMPMIN is our comparison
;
Chk_T1_Min:     cmpa    #TEMPMIN          ;Have we reached the min temp
                bcc     Chk_T2_Max        ;No, look at Sensor #2
                bset    ch_status, 82h    ;Update status word
                bra     Set_FaultMode     ;Now in fault mode

;  Check for Temperature sense 2 (Tsens2) against Temperature Maximum to see
;  if we've exceeded TEMPMAX.  If so, set up as a fault.
;
;  Tsens2 > TEMPMAX is our comparison

Chk_T2_Max:     ldaa    t2_filt           ;Load the filtered temperature
                cmpa    #TEMPMAX          ;Have we reached the max temp
                bcs     Chk_T2_Min        ;No, look at the min
                bset    ch_status, 12h    ;Update status word
                bra     Set_FaultMode     ;Now in fault mode

;  Check for Temperature sense 2 (Tsens2) against Temperature Minimum to see
;  if we've fallen below TEMPMIN.  If so, set up as a fault.
;
;  Tsens2 < TEMPMIN is our comparison
;
Chk_T2_Min:     cmpa    #TEMPMIN          ;Have we reached the min temp
                bcc     Chk_ToFault       ;No, look at next fault
                bset    ch_status, 08ah   ;Update status word
                bra     Set_FaultMode     ;Now in fault mode

;  Check for Time since power-up against Maximum allowable time to charge to
;  see if we're doing okay.  If not, set up as a fault.
;
;  check for Time since power up > TMAX
; 
Chk_Tofault:    ldaa    one_hour_count    ;See how many hours it's been
                cmpa    #TMAX             ;Has the max limit been reached
                bls     Chk_MaxAmpLim     ;No, check the Amphour Counter
                bset    ch_status, 22h    ;update status word
                bra     Set_FaultMode     ;We're now in fault mode

;  Check the amount of amp hours (using amp mins) applied to see if the maximum
;  limit has been exceed.  If so, set as a fault.
;
;  Check Tot_Amp_Mins > Max_Amp_Mins
;
Chk_MaxAmpLim:  ldd     Tot_Amp_Mins      ;See how many amp mins it's been
                cpd     #MAX_AMP_MINS     ;Has the max limit been reached
                bls     Chk_OtFault       ;No, check for next error
                bset    ch_status, 46h    ;update status word
                bra     Set_FaultMode     ;We're now in fault mode

;  Check the Charge Port temperature discrete (PA1) for an overheat or failure
;  condition.  If the discrete is set, we have a failure.
;
Chk_OtFault:    ldx     #REGBAS           ;Get the register base address
                brclr   PORTA,x, 02h, chk_fault_end   ;Looks clean
                bset    Ch_Status, 42h    ;update status word

;  This is where we enter Fault Mode based on a reported failure.
;
Set_FaultMode:  bsr     Fault_Mode        ;Put us into Fault Mode

;  This routine sets us up to exit the interrupt.
;
Chk_Fault_End:  ldx     #REGBAS           ;Offset to registers
                bclr    TFLG1,x,OC2_CLEAR ;Clear IRQ
                rti                       ;Return from the interrupt

;****************************************************************************


;****************************************************************************
;  Fault_Mode    19 Apr 95 by J.W. Hagen, HTSC
;  Description:  This routine controls the entry into Fault Mode.
;  Prog flow:    Linear.
;  Regs Altered: A
;  Global Read:  None.
;  Global Write: Power,Ch_Command
;  Passed value: None.
;  Return value: None.
;
;  This is where Fault Mode is entered based on a reported failure.
;
Fault_Mode:     ldaa    #MODE255          ;Enter Fault Mode
                staa    mode              ;Update the mode
                bset    Flag1,010h        ;Update mode transition flag
                clr     ch_command        ;request 0% pwr in fault mode
                clr     power             ;Power Msb = 0
                clr     power+1           ;Power Lsb = 0
                bset    flag4,040h        ;Show Fault Mode Entered
                rts                       ;Back to the calling program

;****************************************************************************


;  Required modes have been processed.  Set the system to show the charging
;  cycle has been completed successfully.
;
ChargeComplete: ldaa    #1                ;Select SCM message one
                staa    ch_status         ;Send message as charging complete
                ldaa    #100              ;To show 100% complete
                staa    ch_state          ;state of charge = 100%
                clr     ch_command        ;charge command = 0
                clr     power             ;Power Msb = 0
                clr     power+1           ;Power Lsb = 0
                bset    flag4,080h        ;Show Completion Mode Entered
Control_End:    rts                       ;Back to TOF_IRQ

;****************************************************************************
;  control       Scheduled to happen every  (32*32.77ms)=1.048sec
;
;  Description:  Controls output given a particular input. The algorithm
;                uses five modes: constant current #1, constant voltage #1,
;                constant current #2, constant voltage #2, constant current #3.
;                At the beginning of the cycle (large currents) the power is
;                controlled so that a constant charging current is applied.
;                After the charging current has relaxed (ITRIP) the power
;                shall be controlled so that a constant charging voltage is
;                applied.  After the voltage has increased to some max value,
;                VTRIP, during constant voltage mode, the previous two cycles
;                are repeated with different values for voltage & current trip
;                points.  The final constant current mode allows multiple
;                levels to be used, based on a value preset by the customer in
;                the Customer.h file.
;
;                Modified to slow down the approach to set points.  This will
;                prevent an overshoot which occurs when the customer removes
;                the paddle and immediately reinserts it.  10 May 95  JWH
;
;  Prog flow:    Called from tof_irq, linear execution. Call to read_disc.
;  Regs Altered: x, a, b.
;  Global Read:  v_filt, i_filt, disc_status.
;  Global Write: ch_command, ch_status, ch_state
;  Return value: Results of this routine are J1850 data words.
;
;                 Moved I_COR & V_COR to Customer.H file.  11 Mar 95  JWH
;
;                    ===============================
;                    CRITICAL INTEGRATION PARAMETERS
;                    ===============================
;
;  When calculating trip points be sure and use appropriate scale factors that
;  are derived from the specific charger controller running the intended
;  firmware.
;
;  The current scale factor is derived in the following manner :
;
;                              1A  *  Rs(ohm)              255 lsb
;   current scale factor   =  ----------------- * 100 *  -----------
;                                    1A                      5  V
;
;     where Rs = current shunt resistor (Ohms)
;
;     For a 2 mOhm shunt resistor , current scale factor= 10.2 lsb/A
;
;  NOTE: The 10.2 lsb/A has been tweaked to 10.5.  This was accomplished using
;        a Fluke 79 with a current probe.  Also, this tweaking brings the
;        readings displayed in the diags program to approximately +/- 1 amp.
;
; The voltage scale factor is derived in the following manner :
 
;                               1V *  R3                255 lsb
;  voltage scale factor =     --------------------- *  ---------
;                               1V * (R1 + R2 + R3)       5  V
 
;        where R1,R2,R3 = three resistors comprising VBATT breakdown
;                         circuitry in voltage sense branch of charger
;                         controller.
 
;   For R1=R2= 432 KOhm  &  R3= 8.45 KOhm ,
;   voltage scale factor= .4940 lsb/V (nominal value)
;                       = .4928 ( 1st 2nd gen production unit )
;                       = .4983 Based on Fluke 79 and calibration for diags
;                               program.
;
; The following two parameters are the constant current & constant voltage
; gain parameters (I_COR & V_COR).  Two equations describe the dependence
; of these gain terms on various charging system parameters.  Specific
; charging system parameters yield a range throughout which charging system
; will be stable.  If desired gain falls outside range, system stability
; is in jeopardy.
;
;                               VBATT
; [1]     0  <  I_COR   <   -----------------
;                             10 * PMAX * Rs
;
;
;                            10 * VBATT * ( R1 + R2 + R3 )
; [2]     0  <  V_COR   <   --------------------------------
;                             PMAX * Rbatt * R3
;
;
;      where    VBATT = instantaneous traction battery voltage (VDC)
;               PMAX  = maximum charger power (W)
;               Rs    = current shunt resistor (Ohms)
;               R1,R2,R3 = three resistors comprising VBATT breakdown
;                          circuitry in voltage sense branch of charger
;                          controller.
;               Rbatt = traction battery impedance
 
; In the implementation of the original 2nd generation charging algorithm,
; the following assumptions were used :
;
;               VBATT = 400 VDC
;               PMAX  = 6600 W
;               Rs    = 2 mOhm
;               R1,R2,R3 = 432K, 432K, 8.45K (Ohm) respectively
;               Rbatt = 1 Ohm
 
; The above assumptions yield the following allowable ranges for
; I_COR & V_COR :
 
;               0 < I_COR <= 3    unscaled

;               0 < I_COR <= 48   scaled by 2^4
 
;               0 < V_COR <= 62   unscaled
;               0 < V_COR <= 992  scaled by 2^4
;               (maximum V_COR allowed is 255)
;
; When scaling for added gain control resolution use the upper limit
; of the allowable range for unscaled gain (max_gain_unscaled), obtained
; from either relation [1] or [2], and calculate scale factor (SF) under the
; following constraints :
;
;    [3]       max_gain_unscaled * SF --> 255      for highest resolution
;
;    [4]       SF = 2^N    for easily implementable scale down to ch_command
 
 
; Example :    let max_gain_unscaled = 3
;               then SF = 255/3 = 85
;
;                        SF = 2^N
;                   log(85) = N log(2)
;                         N = 6.4
;                   use   N = 6              to update integration parameter
;              therefore SF = 2^6 = 64
;
;              range is 0 < K < (2*64)
;                       0 < K < 128          define range for gain
 
;     overflow limit = 2^(8 + N)
;                    = 2^13
;                    = 4000 Hex

MODEDELAY_MAX:  equ     59                ;59 Second delay between Modes 2 & 3,
                                          ; and Modes 4 & 254

Control:        ldaa    v_filt            ;Get voltage
                staa    v_ctrl            ; to ensure const thru module
                staa    v_ctrl2           ; to ensure const thru module

                ldaa    i_filt            ;Get current
                staa    i_ctrl            ; to ensure const thru module

; Control starts here
;
;  The Get_Parm_Add routine determines the present mode and address for the
;  mode's parameter table.  Mode is returned in the A and the parameter table
;  address is returned in the X register.
;
                jsr     Get_Parm_Add      ;Get the parameter table address

                cmpa    #Mode1            ;Is it mode 1
                beq     Cc_Proceed        ;Yes, we're Constant Current #1

                cmpa    #Mode2            ;Is it mode 2
                beq     Cv_Mode2          ;Yes, we're Constant Voltage #1

                cmpa    #Mode3            ;Is it mode 3
                beq     Cc_Mode3          ;Yes, we're Constant Current #2

                cmpa    #Mode4            ;Is it mode 4
                bne     Control1          ;No, continue checking
                jmp     Cv_Proceed        ;Yes, we're Constant Voltage #2

Control1:       cmpa    #Mode254          ;Completion Mode (Mode 254)
                beq     ChargeComplete    ;Yes, charging is complete

                rts                       ;Must be in Fault Mode, no control

;  We're in a Constant Current Mode (1 or 3)
;
;  If the AMPHR_TRACK_3 flag is set, then trip from Mode 3 when the cumulative
;  amp hours applied during Modes 1 & 2 equal or exceed AMPHR_TRIP_3 as defined
;  in the Customer.h file.
;
Cc_Mode3:       brclr   Flag4,01h,Cc_Mode3a  ;Not tracking Mode 1&2 Amphrs

;  Presently in Mode 3 and the AMPHR_TRACK_3 flag is on.  Determine if AmpHrs
;  applied in Modes 1 & 2 meet or exceed AMPHR_TRIP_3 as defined in the
;  Customer.h file.  If so, trip to Mode 4.
;
;  Formula used for amphr tracking and required percentage returned is as
;  follows:
;
;        ((AmpMinsMode1 + AmpMinsMode2) / 30) * %Desired)
;        ------------------------------------------------      * 3
;                                10
;
                pshx                      ;Save the X on the stack

                ldd     mode1_amp_mins    ;Amp minutes applied while in Mode 1
                addd    mode2_amp_mins    ;Amp minutes applied while in Mode 2
                ldx     #30               ;Divisor
                idiv                      ;Mode1AmpMins / 30
                std     temp_ram3
                xgdx                      ;Move quotient to the D

                psha                      ;Save MSByte for now
                ldaa    #AMPHR_TRIP_3     ;Desired percentage to be returned
                mul                       ;LSByte * % Desired
                std     temp_ram1         ;Save the result
                pula                      ;Restore MSByte
                ldab    #AMPHR_TRIP_3     ;Desired percentage to be returned
                mul                       ;MSByte * % Desired
                addd    temp_ram1         ;Add in the (LSByte * % desired)

                ldx     #10               ;Divisor
                idiv                      ;((AmpMins/30)* % desired) / 10
                xgdx                      ;Put quotient in the D

                psha                      ;Save MSByte for now
                ldaa    #3                ;To normalize data
                mul                       ;LSByte * Normalizer
                std     temp_ram1         ;Save the result
                pula                      ;Restore MSByte
                ldab    #3                ;To normalize data
                mul                       ;MSByte * Normalizer
                addd    temp_ram1         ;Add in the (LSByte * % desired)
                std     temp_ram2         ;This is present amount completed

                pulx                      ;Restore the X register
                cpd     mode3_amp_mins    ;Subtract Amp mins applied in Mode 3

;  If the amount of amp minutes applied in Mode 3 is greater than or equal to
;  the amount (AMPHR_TRIP_3) specified by the customer in the customer.h file
;  then authorize the exit to Mode 4.
;
                bls     Enter_New_Mode    ;Enough amp hours have been applied

;  Mode 3 is also checked against the remaining levels, if applicable.
;
Cc_Mode3a:      brset   flag2,40h,Cc_Proceed   ;Set indicates previous entry
                bset    flag2,40h         ;Set flag to indicate intial entry
                tst     levelsremain      ;Did the customer enter anything
                beq     Cc_Proceed        ;No, don't cause an error
                dec     levelsremain      ;To show this one in progress

;check measured value against trip point
;
Cc_Proceed:     ldy     #v_ctrl           ;Point to ram loc for measured value
                ldaa    0,y               ;RegA = v_ctrl
                cmpa    1,x               ;Check against trip point
                blo     Do_Ctrl           ;If v_ctrl < VTRIP do control

;  If this is Mode 3, other parameters need to be checked before changing to
;  the next mode.  All levels, if provided by the customer, must be processed.
;
                bsr     Check_Mode3       ;Determine if it's okay to chg Modes
                brclr   flag3,10h,Do_Ctrl1   ;Mode change not authorized if
                                             ;the flag is clear
                bra     Enter_New_Mode    ;Mode exit condition met
 
;  We're in a Constant Voltage Mode 2
;
;  If the AMPHR_TRACK_2 flag is set, then trip from Mode 2 when the amount of
;  amp hours applied during Mode 2 equal or exceed AMPHR_TRIP_2 as defined in
;  the Customer.h file.
;
Cv_Mode2:       brclr   Flag2,01h,Cv_Proceed    ;Not tracking Mode1 Amphr
                ldy     #i_ctrl           ;point to ram loc for measured value

;  Presently in Mode 2 and the AMPHR_TRACK_2 flag is on.  Determine if AmpHrs
;  applied in Mode 2 meet or exceed AMPHR_TRIP_2 as defined in the Customer.h
;  file.  If so, check the time delay and if ok, trip to Mode 3.
;
;
;  Formula used for amphr tracking and required percentage returned is as
;  follows:
;
;        ((AmpMinsModeN / 30) * %Desired)
;        ---------------------------------   * 3
;                       10
;
                pshx                      ;Save the X on the stack


                ldd     mode1_amp_mins    ;Amp minutes applied while in Mode 1
                ldx     #30               ;Divisor
                idiv                      ;Mode1AmpMins / 30
                std     temp_ram3
                xgdx                      ;Move quotient to the D

                psha                      ;Save MSByte for now
                ldaa    #AMPHR_TRIP_2     ;Desired percentage to be returned
                mul                       ;LSByte * % Desired
                std     temp_ram1         ;Save the result
                pula                      ;Restore MSByte
                ldab    #AMPHR_TRIP_2     ;Desired percentage to be returned
                mul                       ;MSByte * % Desired
                addd    temp_ram1         ;Add in the (LSByte * % desired)

                ldx     #10               ;Divisor
                idiv                      ;((AmpMins/30)* % desired) / 10
                xgdx                      ;Put quotient in the D

                psha                      ;Save MSByte for now
                ldaa    #3                ;To normalize data
                mul                       ;LSByte * Normalizer
                std     temp_ram1         ;Save the result
                pula                      ;Restore MSByte
                ldab    #3                ;To normalize data
                mul                       ;MSByte * Normalizer
                addd    temp_ram1         ;Add in the (LSByte * % desired)
                std     temp_ram2         ;This is present amount completed

                pulx                      ;Restore the X register
                cpd     mode2_amp_mins    ;Subtract Amp mins applied in Mode 2

;  If the amount of amp minutes applied in Mode 2 is greater than or equal to
;  the amount (AMPHR_TRIP_2) specified by the customer in the customer.h file
;  then perform the Check_Bypass.
;
                bcs     Check_Bypass      ;Enough amp hours have been applied

;  We're in a Constant Voltage Mode (2 or 4).  check measured value against
;  the trip point
;
Cv_Proceed:     ldy     #i_ctrl           ;point to ram loc for measured value
                ldaa    0,y               ;regA= i_ctrl
                cmpa    1,x               ;check against trip point
                bhi     do_ctrl           ;if i_ctrl > ITRIP do control
                                          ;else mode exit condition met

;  If this mode is set for 0 hours and 0 mins, no need to check for a delay.
;  Prevents holding in Mode 2 or Mode 4 when the time is set to zero hours and
;  zero minutes.
;
                tst     0,x               ;Required hours this mode
                bne     Check_Bypass      ;Required hours > 0
                tst     4,x               ;Required minutes this mode
                beq     Enter_New_Mode    ;Required hours & minutes = 0

;  Check the Mode 2 & Mode 4 Bypass Trip Point flag.  If the flag is set, the
;  trip points are ignored and the Mode duration is controlled by the amount of
;  time set for the respective Mode.  If the flag is not set, check for the
;  proper delay in the Check_Delay routine.
;
Check_Bypass:   brset   Flag3,04h,Do_Ctrl ;Bypass Mode Delay

;  We need to hold mode 2 & 4 at least MODEDELAY_MAX seconds.  This is required
;  for when a brownout occurs.  Only required when the time for Mode 2 or Mode
;  4 is NOT set for zero hours and zero minutes.
;
Check_Delay:    ldaa    mode_delay        ;How long have we been in delay
                cmpa    #MODEDELAY_MAX    ;Have we waited long enough
                blt     Do_Ctrl1          ;Not enough time yet, keep working

;  Conditions to exit this mode have been met.  Determine what the next mode
;  should be, make the change and exit.
;
Enter_New_Mode: jsr     Select_Mode       ;Changing to the next mode
                rts                       ;Back to TOF_IRQ
 
;  Perform the calculations for the next power setting and charge command.
;
do_ctrl:        clr     mode_delay        ;Reset the counter
do_ctrl1:       jsr     ctrl              ;do ctrl
                rts                       ;Back to TOF_IRQ

;****************************************************************************


;****************************************************************************
;  Check_Mode3   31 May 95 - JWH
;
;                This module determines if the present Mode is Mode 3.  If so,
;                the levels remaining must be 0 for a mode change to occur.  If
;                not Mode 3, mode change is authorized.  Flag3, bit 4 is the
;                mode change authorization flag returned to the calling
;                program.
;
;  Prog flow:    Called from CONTROL: and CTRL:. Linear execution.
;  Regs Altered: a, b.
;  Global Read:  mode, levelsremain
;  Global Write: mode3 set point, levelsremain
;  Passed Value: None.
;  Return Value: None.
;
;                Note: flag3, bit 4 (10h) set if authorized to change modes
;
;  If this is Mode 3, other parameters need to be checked.
;
LEVEL_DELAY:    equ     30                ;30 Second delay for each level
;
Check_Mode3:    bclr    flag3,10h         ;Clear Mode change authorized flag

;  If not Mode 3, set the authorization flag and exit.
;
                ldaa    mode              ;Get the present mode
                cmpa    #MODE3            ;Is it Mode 3
                bne     Check_Mode3b      ;No, Mode change is authorized

;  See if the time out flag is set.  If so, indicates too much time in this
;  mode or level.  Time to progress to the next level.
;
                brset   flag3,20h,Check_Mode3a  ;If set, this one timed out.

;  Determine if a hold is in effect for this level.  If not, advance to the
;  next level.
;
                brclr   flag3,40h,Check_Mode3a  ;If clear bypass mode delay.

;  See if time in this level meets or exceeds the LEVEL_DELAY requirement.  If
;  not, level change is not authorized.
;
                ldaa    mode3_delay       ;How long have we been in delay
                cmpa    #LEVEL_DELAY      ;Have we waited long enough
                bcs     Check_Mode3c      ;Not enough time yet, keep working

;  Enough time has been spent in this level.  Clear the flag and exit.  During
;  the next pass in this module, level change will be authorized.
;
                bclr    flag3,40h         ;Clear the mode delay requirement
                rts
                
;  Enough time has been spent in this level.  Clear the mode3_delay counter,
;  check for more levels, decrement the levels remain counter if appropriate,
;  cacluclate the next set point and exit.
;
Check_Mode3a:   clr     mode3_delay       ;Clear the mode3_delay counter.

;  If levels remaining is equal to 0, set the authorization flag and exit.
;
                tst     levelsremain      ;Any levels remaining
                beq     Check_Mode3b      ;No, Mode change is authorized

                ldaa    #AMPSPERLEVEL     ;Amps per level, Mode 3
                ldab    levelsremain      ;Levels remaining
                mul                       ;LevelAdjust*LevelsRemain=New set pt.
                stab    2,x               ;Mode3 Parms (Set Point)
                                          ;Note: Answer is always in the B reg.
                dec     levelsremain      ;Show another level complete
                bclr    flag3,20h         ;Clear the timeout flag
                bset    flag3,40h         ;Enable the mode delay requirement
                rts                       ;No Mode change authorized, return

;  Set the authorization flag to change modes and exit.
;
Check_Mode3b:   bset    flag3,10h         ;Set Mode Change is Authorized flag
                bclr    flag3,20h         ;Clear the timeout flag
Check_Mode3c:   rts                       ;Return to calling program

;****************************************************************************


;****************************************************************************
;  ctrl          Modified to slow down iteration when approaching a set point.
;                Iteration is normal when the set point has been exceeded.  The
;                iteration is slowed when approaching the set point.
;
;                10 May 95  JWH
;
;  Description:  Constant current/ constant voltage control algorithm.
;  Prog flow:    Called from control, linear execution.
;  Regs Altered: a, b.
;  Global Read:  v_ctrl, i_ctrl, mone_hour_count
;  Global Write: ch_command
;  Passed value: address of integration parameters block in reg X
;                address of v_ctrl or i_ctrl in reg Y
;  Return value: None.
;
;                Moved Slew Rate limiters to Customer.h file.  11 Mar 95  JWH
;
;  First check to see if we have exceeded our time limit in this mode before
;  we do anything else.
;
Ctrl:           bclr    Flag1,08h         ;Reset the negative error flag
                ldaa    mone_hour_count   ;How many hours in this mode
                cmpa    0,x               ;check for timeout : hours
                bhi     Ctrl_Ck_Mode      ;enter new mode if
                                          ;mode time(hrs) > timeout (hrs)
                blo     Ctrl_Proceed      ;proceed with ctrl if
                                          ;mode time(hrs) < timeout (hrs)

;  check minutes only when mode time (hrs) = timeout (hrs)
;
                ldaa    mone_min_count    ;How many minutes in this mode
                cmpa    4,x               ;check for timeout : minutes
                blo     Ctrl_Proceed      ;proceed with ctrl if
                                          ;mode time(mins) < timeout (mins)

;  Since this is a time out, set a bit to indicate this mode or level did time
;  out.  Critical for Mode 3 to continue counting down the levels.
;
                bset    flag3,20h         ;Indicate mode/level time out occurred

;  If this is Mode 3, other parameters need to be checked.
;
Ctrl_Ck_Mode:   bsr     Check_Mode3       ;Determine if it's okay to chg Modes
                brclr   flag3,10h,Ctrl_Proceed  ;Mode change not authorized if
                                          ;the flag is clear

;  enter new mode because mode exit condition met
;
Enter_NewMode:  jsr     Select_Mode       ;Changing to next mode
                rts                       ;Back to TOF_IRQ

;  We didn't time out so let's proceed
;
Ctrl_Proceed:   ldaa    2,x               ; regA = set point
                suba    2,y               ; regA = set point - measured val
                                          ;      = error (signed)
                staa    actual_error      ;Remember SetPoint - Measured

                bcc     Ctrl_Gm           ;If set point was > measured, branch
                bset    flag1,08h         ;Had to borrow, set neg error flag
                                          ;(0=False 1=True)
                nega                      ;regA = set point - measured val
                                          ;     = error (unsigned)

                bclr    flag3,08h         ;For positive error calculations

;  Gain control, how's it going
;
ctrl_gm:        ldab    3,x               ;regB = gain
                mul                       ;regD=error * gain
                std     ctrl_temp         ; ctrl_temp= error * gain (unsigned)

;  See if we were in the negative.  If so, gain was okay or we need less.  If
;  not, we need more.
;
                brclr   flag1,08h,Ctrl_Pos   ;If error >=0, then branch
 
;  error < 0 so we need less gain
;
;  Pn   = Power New
;  Pn-1 = Power Old
;
                ldaa    #200              ;Anything larger than sample period
                staa    sample_periods    ;To force an update when it goes pos

                ldaa    actual_error      ;SetPoint - Measured
                cmpa    #5                ;Is SetPoint - Measured within 5 lsb
                bcc     Ctrl_Neg2         ;No, normal calculation

;  Since SetPoint - Measured < 10 lsb's, start decrementing the new power
;  power command by 1.  Do this be changing the previously saved value from
;  SetPoint - Measured.  Force it to equal 1 bit.  This slows down the rate of
;  change by the power command.
;
                ldd     #1                ;Decrement power by 1 bit
                std     ctrl_temp         ;So SetPoint - Measured forced to = 1

;  Double the error
;
Ctrl_Neg1:      ldab    actual_error      ;Get SetPoint - Measured (unsigned)
                lslb                      ;Double the value of the neg error
                bcc     Ctrl_Neg1a        ;Doesn't exceed 255, branch
                ldab    #255              ;Load max value

Ctrl_Neg1a:     ldaa    3,x               ;Normal Gain Factor
                mul                       ;(((SetPoint-Measured)unsigned)2)gain
                std     ctrl_temp         ;Save as the amplified error

Ctrl_Neg2:      ldd     power             ;power = Pn-1
                subd    ctrl_temp         ;regD=Pn=Pn-1 - (error * gain)
                bcs     Ctrl_Neg3
                std     power             ;Save the updated power command
                cpd     #16               ;Is it at least .3% power
                bcc     scale_truncate    ;If power > ctrl_temp, scale

;  Pn-1 - (error*gain) < 0
;
Ctrl_Neg3:      clr     power             ;Saturate Low
                clr     power+1           ;LSByte
                clrb                      ;Clear the charge command
                bsr     BrownOut          ;Check for BrownOut condition
                bra     Update_Ch_Cmd     ;Save as the updated charge command

;  Pn-1 - (error*gain) < 0
 
;  error >= 0
;
Ctrl_Pos:       brclr   Flag3,08h,Ctrl_PosA  ;Had neg error, norm calculations
                inc     sample_periods    ;Increment the delay counter
                ldaa    sample_periods    ;Get the delay counter
                cmpa    #37               ;38 passes * .131msec = 4.978 secs
                bcc     Ctrl_PosA         ;Time to update the charge cmd

                ldd     power             ;Load old power command
                bra     Scale_Truncate    ;Process and save it

Ctrl_PosA:      clr     sample_periods    ;Clear the delay counter

;  Check the flag to see if a negative error has been experienced.  If so, no
;  limiters are in place to help prevent oscillations.
;
                brclr   Flag3,08h,Ctrl_Pos1a ;Had neg error, calc norm

;  Limiters are in place.
;
                ldaa    actual_error      ;SetPoint - Measured
                cmpa    #10               ;Is SetPoint - Measured within 10 lsb
                bcc     Ctrl_Pos1         ;No, normal calculation

;  Since SetPoint - Measured < 10 lsb's, start incrementing the new power
;  power command by 1.  Do this be changing the previously saved value from
;  SetPoint - Measured.  Force it to equal 1 bit.  This slows down the rate of
;  change by the power command.
;
                ldd     #5                ;Increment power by 5 bits
                std     ctrl_temp         ;So SetPoint - Measured forced to = 5
                bra     Ctrl_Pos1a        ;

Ctrl_Pos1:      ldd     ctrl_temp         ;Get the amplified error
                cpd     #48               ;Limit to 1% power change
                bcs     Ctrl_Pos1a        ;Less than 48, it'll work
                ldd     #48               ;Load 1% power equivilant
                std     ctrl_temp         ;Save as 1% power delta

Ctrl_Pos1a:     bset    Flag3,08h         ;Return to restricted changes
                ldd     power             ; power = Pn-1
                addd    ctrl_temp         ; regD= Pn = Pn-1 + (error*gain)
                cpd     #OVERFLOW_LIMIT   ; look for overflow
                bhs     sat_hi            ; if overflow , then saturate hi

;  This is where we perform the scale prior to saving the data
;
Scale_Truncate: bsr     BrownOut          ;Check for a brownout
                bsr     Regulate_Power    ;Check for power limiting
                std     power             ; update state variable
                ldy     #POWER_SCALE      ;regY= N , where scale factor = 2^N
                jsr     Shift_Right_D     ;Perform the necessary truncation
                bra     Update_Ch_Cmd     ;We're done, time to save the data

;  This is where we start to force it into saturation
;
Sat_Hi:         ldd     #OVERFLOW_LIMIT   ;regD=16-bit 'power' overflow
                subd    #1                ;regD=max value, start lowering it
                bsr     BrownOut          ;Check for a brown out
                bsr     Regulate_Power    ;Check for power limiting
                std     power             ;saturate

;  Update the Charge Command and exit.
;
Update_Ch_Cmd:  stab    ch_command        ;truncate ms byte of 16-bit of the
                                          ;'power' variable to get the
                                          ; 8-bit ch_command
Ctrl_Exit:      rts                       ;Finished.

;****************************************************************************


;****************************************************************************
;  BrownOut      12 Dec 94 by J.W. Hagen, HTSC
;  Description:  This routine controls a predetermined timeout when a brownout
;                condition is in progress, loss of SCM J1850 transmitted msgs
;                or the SCM tripped / shut-down.  Also controls recovery from a
;                BrownOut and/or entry into fault mode should recovery not
;                occur within BrownOut_Lim seconds.
;  Prog flow:    Linear.
;  Regs Altered: A
;  Global Read:  Power
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
BrownOut_Max:   Equ     3                 ;Brownout in progress after 3 seconds
BrownOut_Lim:   Equ     30                ;Maximum # of seconds for brownout
BrownOut_Hold:  Equ     5                 ;Maximum # of seconds to recover from
                                          ;a brownout before adjusting power.
;
BrownOut:       psha                      ;Save the A (New Power MSByte)
                pshb                      ;Save the B (New Power LSByte)

                ldaa    brownout_secs     ;Compare to present value
                cmpa    #BROWNOUT_MAX     ;Get the Maximum seconds counter
                blo     BrownOut2         ;Within limits, we're ok
                clr     brownout_wait     ;Clear the BrownOut Restore Counter
                bclr    Flag1,020h        ;Clear SCM J1850 Tx Running OK flag
                bset    Flag1,040h        ;BrownOut Recovery Required Flag

;  See if it's been long enough.  If so, show as Fault Mode.  Error code is
;  016h.
;
                cmpa    #BROWNOUT_LIM     ;Been in brownout too long
                blo     BrownOut1         ;No, still under the limit
                bset    ch_status, 16h    ;Update status word
                jsr     Fault_Mode        ;Enter Fault Mode

BrownOut1:      pulb                      ;Remove old Power LSByte
                pula                      ;Remove old Power MSByte
                ldd     power             ;Restore the Old Power Command (PN-1)
                rts                       ;Back to the calling program

BrownOut2:      brclr   Flag1,040h,BrownOut3 ;Not a previous BrownOut
                ldaa    #BROWNOUT_HOLD    ;Get the required holding time
                suba    brownout_wait     ;How long have we been in recovery
                bcc     BrownOut1         ;Not long enough, use old power level
                bclr    Flag1,040h        ;Clear previous BrownOut flag

BrownOut3:      clr     brownout_wait     ;Clear the BrownOut Restore Counter
                pulb                      ;Restore New Power LSByte
                pula                      ;Restore New Power MSByte

BrownOut4:      rts                       ;Back to the calling program

;****************************************************************************


;****************************************************************************
;  Regulate_Power 26 Apr 95 by J.W. Hagen, HTSC
;  Description:   This routine determines if the calculated power command will
;                 cause current to exceed the value preset in either IMAX_MODE2
;                 or IMAX_MODE4.  If so, the old power command is restored and
;                 used instead of the new calculation.  This applies only to
;                 Modes 2 & 4.  Modes 1 & 3 return without modification.
;  Prog flow:     Linear.
;  Regs Altered:  D
;  Global Read:   Mode
;  Global Write:  None.
;  Passed value:  Power (newly calculated command).
;  Return value:  Power (either old or original upon entry).
;
Regulate_Power: psha                      ;Save the A (New Power MSByte)
                pshb                      ;Save the B (New Power LSByte)

                ldaa    mode              ;What mode is this
                cmpa    #MODE4            ;Is it Mode 4
                beq     Regulate_Mode4    ;Yes, process current regulator

                cmpa    #MODE3            ;Is it Mode 3
                beq     Regulate_Mode3    ;Yes, process voltage regulator

                cmpa    #MODE2            ;Is it Mode 2
                beq     Regulate_Mode2    ;Yes, process current regulator

;  Must be Mode 1
;
;  Determine if the new power command will cause the Vsense or Isense to exceed
;  either VMAX_MODE1, IMAX_MODE2, VMAX_MODE3, or IMAX_MODE4 as appropriate.
;
;  Mode 1.  Test against VMAX_MODE1 limit
;
Regulate_Mode1: ldaa    v_filt            ;Get the present voltage
                cmpa    mode1_trippt      ;Compare to the trip point
                bra     Regulate_1_3      ;Determine power command validity

;  Mode 2.  Test against IMAX_MODE2 limit
;
Regulate_Mode2: ldaa    i_raw             ;Use the raw current
                cmpa    #IMAX_MODE2       ;How does it compare
                bra     Regulate_2_4      ;Determine power command validity

;  Mode 3.  Test against VMAX_MODE3 limit
;
Regulate_Mode3: ldaa    v_filt            ;Get the present voltage
                cmpa    mode3_trippt      ;Compare to the trip point
Regulate_1_3:   bcs     Regulate_New      ;Raw < Trip, new power command valid
                bra     Regulate_Volts    ;Too high, use old power command

;  Mode 4.  Test against IMAX_MODE4 limit
;
Regulate_Mode4: ldaa    i_raw             ;Use the raw current
                cmpa    #IMAX_MODE4       ;How does it compare
Regulate_2_4:   blo     Regulate_New      ;Looks okay, new power command valid

;  The new power command will cause the current to exceed the defined limit.
;  Restore the old power command and exit.
;
Regulate_Amps:  pulb                      ;Remove old Power LSByte
                pula                      ;Remove old Power MSByte
                ldd     power             ;Restore the Old Power Command (PN-1)
                rts                       ;Back to the calling program

;  The new power command may cause the voltage to exceed the defined limit.
;  Restore the old power command, if it's less than the new one, then use it
;  for the new power command and exit.  If the new power command is less than
;  the old one, use it since it will reduce the voltage.
;
Regulate_Volts: pulb                      ;Remove new Power LSByte
                pula                      ;Remove new Power MSByte
                ldd     power             ;Restore the Old Power Command (PN-1)
                subd    #08h              ;8 lsb's = .5% ch_cmd reduction
                bhi     Regulate_Rts      ;Still have power left, continue
                ldd     #08h              ;To have at least .5% charge command
                rts                       ;Back to the calling program

;  The new power command has been determined to be valid.  Restore the new
;  power command and exit.
;
Regulate_New:   pulb                      ;Restore New Power LSByte
                pula                      ;Restore New Power MSByte
Regulate_Rts:   rts                       ;Back to the calling program

;****************************************************************************


;****************************************************************************
;  dlcs_irq      Modified 13 Dec 94 by JWH for detection of SCM xmit msgs
;
;  Description:  Data Link Controller Serial (DLCS) IRQ routine.  Called on:
;                1. An End Of Data (EOD) has been detected and CC byte is in
;                RFIFO.
;                2. The Receiver First In First Out (RFIFO) has 12 bits in it
;                and the 13th byte comes in
;                3. Tx in progress, no last byte, buffer half empty
;                4. DLCS wakeup (not implemented)
;                This routine checks for recieved data and empties the RFIFO
;                storing data to the rx_buffer, status to rx_status (single
;                buffered), and the completion code to rx_ccode.  It also
;                places the length of the currently recieved msg in rx_size.
;  Prog flow:    IRQ service.  Multiple calls to putdlcs.
;  Regs Altered: x, a, b.
;  Global Read:  None.
;  Global Write: rx_status, rx_buffer, rx_ccode, rx_size, bad_ccode            .
;  Return value: None.
;
DLC_DUMMY:      equ     00h               ;DLCS Dummy Data
DLC_RX_CMD:     equ     02h               ;DLCS Command:
                                          ;CMD: 0000,0010  Rx flush

Dlcs_Irq:       ldx     #rx_buffer        ;Load start address of buffer
                clr     rx_size           ;Clear message size
Dlcs_Irq1:      ldaa    #DLC_DUMMY        ;Load Data (Dummy)
                ldab    #DLC_RX_CMD       ;Recieve/flush command
                jsr     putdlcs           ;Send message to DLCS
                staa    rx_status         ;Store status
                anda    #0e0h             ;AND stat w/ 1110,0000 Rx mask
                beq     dlcs_irq4         ;If not Rx info, go away
                cmpa    #0e0h             ;Check if CC only (last byte)
                beq     dlcs_irq2         ;Go store it and leave
                stab    0,x               ;Store data to buffer
                inx                       ;Bump buffer pointer
                inc     rx_size           ;Bump rx size data
                bra     dlcs_irq1         ;Go get more data

;  Since it was a receive message, check to see if there were any errors
;  detected.
;
Dlcs_Irq2:      stab    rx_ccode          ;Save completion code
;               andb    #80h              ;CC error bit mask
;               beq     dlcs_irq3         ;Beq if no error detected
;               ldaa    rx_ccode          ;Get completion code
;               staa    bad_ccode         ;Save faulty condition code

;  Determine if the receive message was from this node (Charge Controller) or
;  from another node (SCM).  If the Rx-Ccode has bits 4 & 5 set, then the
;  message originated from this node.  If not, then it was from another node.
;
Dlcs_Irq3:      ldaa    rx_ccode          ;Get the completion code
                anda    #30h              ;Transmitter successful (this node)
                cmpa    #30h              ;Is it from this node?
                beq     dlcs_irq4         ;Yes, it's not what we're looking for

;  Now that we know we have an actual message from the SCM, set the flag to
;  show the SCM J1850 is transmitting.  Then clear the brownout sec counter.
;
                bset    Flag1,020h        ;Set flag to show SCM J1850 Tx ok
                clr     BrownOut_Secs     ;Clear the J1850 Out, seconds counter

;  If this message is a "Wake Up" request, capture the max & min power
;  available from this charger.
;
;               ldx     #Rx_Buffer        ;Address of the J1850 receive buffer
;               ldaa    4,x               ;Get the J1850 Secondary ID
;               cmpa    #23h              ;Is it a wake up message
;               bne     dlcs_irq4         ;No, exit

;  This is a wake up request.  Obtain the power rating.
;
;               ldd     5,x               ;Charger Output Power Rating
;               std     SCM_Max_Power     ;Save as the maximum power
;               ldd     7,x               ;Charger Minimum Output Power Rating
;               std     SCM_Min_Power     ;Save as the maximum power
;               bset    Flag2,10h         ;Set to show SCM Output Pwr Known

Dlcs_Irq4:      rti                       ;Return from the interrupt

;****************************************************************************


;****************************************************************************
;  Temperature Compensated Voltage - 01 Jun 95 by J.W. Hagen, HTSC
;  Description:   Calculates voltage offsets based on temperature and applys
;                 these offsets to either V1, V2, Vtrip1, or Vtrip2
;                 based on the present mode.
;  Prog flow:    Linear execution
;  Regs Altered: D,X,Y
;  Global Read:  V1, V2, Vtrip1, Vtrip2
;  Global Write: Modex_parms where x = 1, 2, 3, or 4.
;  Passed value: None.
;  Return value: None.
;
;                       G A T E S   B A T T E R Y
;
;                    S E A L E D   L E A D   A C I D
;
;  The temperature compensation offset voltage is determined in the following
;  manner:
;
;     temp range      = -30 degree to +60 degree celsius (90 degree C spread)
;     where 1.28 lsb  = 2.6v * .4928 lsb/v = gassing voltage per cell at -30
;                       degrees celsius
;     where .0022 lsb = 4.44 mv = (.4 / 90) (slope of gassing voltage curve)
;                       also known as Temp_Comp_Mv which is preset in the
;                       customer.h file.  The .0022 used here is an example.
;     where 2.11 lsb  = per degree Kelvin
;     where Tn_Temp   = T1_filt or T2_filt (with TempMin already subtracted)
;     where 629 lsb   = 2.11 lsb * 298 Kelvin = 25 degrees C
;     where 513 lsb   = 2.11 lsb * 243 degrees kelvin (min temperature which
;                       is measurable by the temperature sensors.)
;     where 6         = Number of Cells per battery
;     where 27        = Number of Batteries in series
;     where 162       = TEMP_BATT_INFO = Number of Cells * Number of Batteries
;     where v/lsb     = volts per lsb (i.e. .4983 v/lsb)
;
;                    (4.44 mv * v/lsb)
;  - v_temp_offset = ----------------- * ((Tn_Temp + 513)-629) * TEMP_BATT_INFO
;                         2.11 lsb
;
;                      .002212452 lsb
;  - v_temp_offset =  ---------------  * (Tn_Temp - 116) * TEMP_BATT_INFO
;                         2.11 lsb
;
;  - v_temp_offset =  .001048555 lsb * (Tn_Temp - 116) * TEMP_BATT_INFO
;
;  - v_temp_offset =  .169865982 lsb * (Tn_Temp - 116)
;
;     Since we can't code fractions, we'll factor the entire fraction by 256
;     (2^8).   This allows us to take all numbers to integers.
;
;  - v_temp_offset =  43 * (Tn_Temp - 116)
;                    -----------------------
;                             256
;
;  With all of that said and done, let's look at an example.  We'll use a
;  temperature reading of 27 degrees celsius, 27 batteries and 6 cells.
;
;  The charge controller temperature sensors read voltage from 0 - 5 which
;  represents a temperature range of 245 K to 365 K.  Each lsb from the 5v
;  equals .019607843 volts (5v / 255 bits).  Since the temperature range has a
;  spread of 120 degrees, each lsb from the 5v equals .041666666 degrees Kelvin
;  (5v / 120 k).  For our example of 25 degrees C, our sensor will return a
;  value of 117 (decimal) which was determined as follows:
;
;     300 K = 27 degrees C
;     300 - 245 (temp min) = 55 K
;     55 K * .041666666 = 2.29166663 volts
;     2.29166663v / .019607843 v/lsb = 117 reading from the controller.
;
;  - v_temp_offset =    43 * (117 - 116)
;                      ------------------
;                             256
;
;  - v_temp_offset =    43 * 1
;                      --------
;                         256
;
;  - v_temp_offset = 43 / 256
;
;  - v_temp_offset = .16796875 
;
;
;  Local Variables and storage
;
TEMP_OFFSET:    equ     513               ;Min reading from temp sensor in lsb
TEMP_KELVIN:    equ     273               ;273 Kelvin = Zero Degrees C
TEMP_BATT_INFO: equ     {Batteries*Cells} ;Batteries * Cells
TEMP_CONSTANT1: equ     {{{{{TEMP_COMP_MV*VOLTS_SCALE}/LSB_PER_KELVIN}*TEMP_BATT_INFO}*256}/10000000}+1
TEMP_CONSTANT2: equ     {{{TEMP_NOMINAL_C+TEMP_KELVIN}*LSB_PER_KELVIN}/100}-TEMP_OFFSET

;  If the temperature compensation bit is set, perform temperature compensation
;
Temp_Comp:      brclr   flag1,01h,Temp_RTS ;Flag clear, no temperature comp.

                bclr    flag2,20h         ;Clear the Temp Comp Neg Offset Flag

;  Temp Comp is enabled, let's get started.  Look for the highest temperature.
;
                ldaa    t1_filt           ;Get the temperature for Sensor 1
                cmpa    t2_filt           ;How does it compare to Sensor 2
                bhs     Temp_Comp2        ;Sensor 1 has a higher temperature
                ldaa    t2_filt           ;Load the temperature for sensor 2

;  Next, subtract the 116 lsb's to get us to a known point, 25 degrees C.
;  If it's a negative value, set a flag so we'll know later.
;
Temp_Comp2:     suba    #temp_constant2   ;Tn_Temp - 116 offset
                bcc     Temp_Comp3        ;Result was not a negative number
                nega                      ;So we're with a positive number
                bset    flag2,20h         ;Set a flag due to the negative #

Temp_Comp3:     ldab    #temp_constant1   ;Our multiplier
                mul                       ;43 * (Tn_Temp - 116)

;  We need to scale down to normalize the data.  Our temp_constant1 was scaled
;  by 2^8 (256) so it's time to normalize the offset so we can apply it.
;
                clc                       ;Clear the carry bit
                ldy     #08h              ;For our scaling factor
                jsr     shift_right_d     ;Divide by 256

;  The offset voltage is now in the B accumulator.  If it's a negative value,
;  when it's added to our Vset or Vn, it'll correct itself.  What we need to
;  know is if it was a negative number at one time.  If so, don't negate it.
;  If not, we will negate it because as temperature goes up, we want the offset
;  to be a negative offset.  Temperature goes down, our offset goes up.
;
                brset   Flag2,20h,Temp_Comp4
                negb                      ;Get the 2's compliment

Temp_Comp4:     stab    v_temp_comp       ;Save the LSByte, our offset voltage

;  So now we know what the voltage offset should be, let's determine the mode
;  we're in and calculate the trip or set point by adding the offset.
;
                jsr     Get_Parm_Add      ;Get the present mode and parameter
                                          ;table starting address.
                                          ; A reg = Mode, X reg = param table
                cmpa    #Mode1            ;Mode 1, Constant Current
                beq     temp_vtrip1       ;Yes, change the Vtrip1 setting
                cmpa    #Mode2            ;Mode 2, Constant Voltage
                beq     temp_v1           ;Yes, change the V1 setting
                cmpa    #Mode3            ;Mode 3, Constant Current
                beq     temp_vtrip2       ;Yes, change the Vtrip2 setting
                cmpa    #Mode4            ;Mode 4, Constant Voltage
                beq     temp_v2           ;Yes, change the V2 setting
                rts                       ;Finished here.

;  Set up the Y register to point to the address of the voltage setting that
;  we'll be modifying.  Set the X register to the begining of the parameters
;  for this mode.  It will be used to look at the time required.
;
Temp_Vtrip1:    addb    #VTRIP1           ;Add in VTRIP1
                ldy     #mode1_parms+1    ;Our destination Address
                bra     Temp_Comp5        ;Check the Hi/Low

Temp_V1:        addb    #V1               ;Add in the offset
                ldy     #mode2_parms+2    ;Our destination Address
                bra     Temp_Comp5        ;Check the Hi/Low

Temp_Vtrip2:    addb    #VTRIP2           ;Add in the offset
                ldy     #mode3_parms+1    ;Our destination Address
                bra     Temp_Comp5        ;Check the Hi/Low

Temp_V2:        addb    #V2               ;Add in the offset
                ldy     #mode4_parms+2    ;Our destination Address

;  Now that we know what the voltage should be, we need to be sure it falls
;  within the TEMP_MAX_VOLTS and TEMP_MIN_VOLTS range.  The new value for our
;  voltage setting is presently in the B accumulator.  Leave it there until
;  we're sure it's a valid number.
;
Temp_Comp5:     cmpb    #Temp_Max_Volts   ;Compare to Maximum Allowed
                bls     Temp_Comp6        ;Looks okay here, check the min
                ldab    #Temp_Max_Volts   ;Get Maximum Allowed
                bra     Temp_Comp7        ;Now go check the mode time
Temp_Comp6:     cmpb    #Temp_Min_Volts   ;Compare to Minimum Allowed
                bhs     Temp_Comp7        ;Looks ok, we'll use the new value
                ldab    #Temp_Min_Volts   ;Get Minimum Allowed

;  If this mode is set for 0 hours and 0 mins, no need to modify the set point.
;
Temp_Comp7:     tst     0,x               ;Required hours this mode
                bne     Temp_Comp8        ;Required hours > 0, use new SetPoint
                tst     4,x               ;Required minutes this mode
                beq     Temp_Rts          ;Required hours & minutes = 0

;  We finally made it.  Save our voltage setting where it belongs and we're
;  back in business.
;
Temp_Comp8:     stab    0,y               ;Save as the new trip point

Temp_Rts:       rts                       ;We're outta here

;****************************************************************************


;****************************************************************************
;  Calculate State Of Charge - 24 May 95 by J.W. Hagen, HTSC
;  Description:  Calculates the approximate state of charge for the battery
;                pack.  Calculates the number of Amp Minutes and Amp Hours
;                applied to the battery for this charging cycle.
;  Prog flow:    Linear execution
;  Regs Altered: D,X
;  Global Read:  ch_state, tot_amp_mins
;  Global Write: ch_state, tot_amp_mins
;  Passed value: None.
;  Return value: None.
;
;  Battery state of charge is calculated based on amp-hours, Vsense at the
;  start of the charging cycle, and Isense during the charging cycle.  The
;  following is our formula to determine the State of Charge (SOC):
;
;              (99) (Isense) (Ttotal)     Volt_Start - (11.58)(#Batt)
;     SOC   =  ----------------------- + -----------------------------
;                    (AH) (1.2)                 (1.2) (#Batt)
;  where:
;     99          = % Battery Charge Capacity
;     Isense      = Present Isense reading
;     Ttotal      = Total time elapsed since power-up (Modes 0 thru 5)
;     Volt_Start  = Vsense reading before entering Mode 1
;     11.58       = 0% State of Charge Open Circuit Voltage Reference Point
;     #Batt       = Number of Battery modules in series string
;     1.2         = 120% of Energy Returned
;     AH          = AmpHour capacity of Battery Pack
;     Zero_Ref    = 0% State of Charge Open Circuit Voltage Reference Point
;
;  Let's look at an example equation with the following known values:
;
;     Isense      = 20 amps
;     Ttotal      = .9 hours
;     Volt_Start  = 321 volts = 160 lsb / .4867 v/lsb
;     AH          = 60
;
;              (99) (Isense) (Ttotal)     (Volt_Start - (11.58)(#Batt)) * 100
;     SOC   =  ----------------------- + ------------------------------------
;                    (AH) (1.2)                       (1.2) (#Batt)
;
;  We'll invert the 99 on the left side of the equation to preclude a
;  requirement for 24 bit math due to the running total of Amp Minutes * 99
;  could exceed 65,535 (FFFFh).
;
;               (20) (.9)              (321 - (11.58 * 27)) * 100
;     SOC   =  -------------------- + ----------------------------
;               (60) (1.2 * .0101)               (1.2) (27)
;
;
;               (20) (.9)              (321 - (11.58 * 27)) * 100
;     SOC   =  -------------------- + ----------------------------
;               (60) (.012)                      (1.2) (27)
;
;
;                18      834
;     SOC   = ------- + ------
;               .72      32.4
;
;        NOTE: As you can see above, we now have a situation where we have
;              fractions involved in the equation.  During the actual execution
;              of the code, we'll be scaling up by a factor of 2^7 to preculde
;              working with fractions.
;
;     SOC   = 25.00 + 25.74
;
;     SOC   = 50%
;
Zero_Ref:       equ     {{{BATTERIES*1158}*VOLTS_SCALE}/VOLTS_MULTIPLY}/100
                                          ;(Number of Batteries*11.58)* V/lsb

Batt_Energy:    equ     {{{BATTERIES*120}*VOLTS_SCALE}/VOLTS_MULTIPLY}/100
                                          ;(Number of Batteries*1.20) * V/lsb

Amp_Energy:     equ     {AMP_HOURS*155}/100
                                          ;(AH * 1.2 * (1/99)) * 2^7
                                          ;  Scaled 2^7 for resolution

;  Perform calculation only if in Mode 1, 2, 3, or 4.
;
Calc_Soc:       ldaa    mode              ;Get the present mode
                cmpa    #MODE254          ;See where we're at
                bcc     Calc_Soc_Exit     ;We're in Completion or Fault Mode

;  Next see if we are just starting a new minute.
;
                tst     one_sec_count     ;Are we at Zero seconds
                bne     Calc_Soc_Clr      ;No, then it's not time.  Clear flag.

;  Now test to see if we have already sampled for this minute.  If so, we'll
;  bail out and wait until the start of a new minute (seconds = 0).
;
                brset   flag1,02h, Calc_Soc_Exit   ;If set, don't process

;  Let's perform the left side of the equation.  The multiplier is set to 100
;  which is the same value used to multiply the Current Scaling factor to pass
;  over to the diags program.  This then allows us to use the Scale_Current
;  variable to determine our actual measured amps at this time.
;
                ldaa    i_filt            ;Get the present current reading
                ldab    #AMPS_MULTIPLY    ;Same as used for Current scaling
                mul                       ;Scale it up
                ldx     #AMPS_SCALE       ;Get the factored Scale_Current
                idiv                      ;Converting to decimal Amps
                xgdx                      ;Isense actual amps is now in the B
                ldaa    #TSAMPLE          ;Our sample interval
                mul                       ;Yields AmpsMin this sample in B

;  At this point we know how many amps we're pulling.  Add this value to the
;  running total for all modes and the running total for this mode.
;
                std     temp_ram          ;Save a copy of present ampmins
                jsr     Get_Parm_Add      ;Fetch the paramater table address
                ldd     temp_ram          ;Get the saved ampmins information
                addd    8,x               ;Add the ampmins to existing total
                std     8,x               ;Save updated ampmins for this mode
                ldd     temp_ram          ;Get the saved ampmins information               
                addd    tot_amp_mins      ;Add in the previous total all modes
                std     tot_amp_mins      ;Update total for all modes

;  Calculate Amp Hours applied to charging the battery thus far.
;
                ldx     #Samples_Hour     ;Sample periods per hour
                idiv                      ;(Tot_Amp_Mins/Periods Per Hour) =
                                          ;  AmpHours Applied to the Battery
                xgdx                      ;Move Amp Hours to the D Accumulator

;  The D accumulator contains the AmpHours applied to our battery pack since we
;  entered Mode 1.  The next step is to determine what percentage this is of
;  the battery capacity (based on 120% capacity).
;
;  In order to complete the left side of the equation we need to scale the Amp
;  Hours by 2^7.  This is because we have scaled the Amp_Energy by 2^7 as well.
;
                ldy     #7                ;Our Number of Left Shifts
                jsr     Shift_Left_D      ;Scale the Amp Hours up by 2^7
                ldx     #AMP_ENERGY       ;Our Amp_Hours * 1.2
                idiv                      ;Yields % increased since power-up
                xgdx                      ;Our % increase is now in the B

;  The left side of the equation is complete.  Now let's get the SOC at Startup
;
                brclr   Flag1,04h,Calc_Soc_1 ;Didn't start with - Soc
                cmpb    soc_at_start      ;See if we've met the deficit
                bls     Calc_Soc_5        ;Still in the negative, exit
                subb    soc_at_start      ;Subtract our deficit
                bra     Calc_Soc_2        ;Continue with the process

;  Now we'll add the right side of the equation to it and we're done.
;
Calc_Soc_1:     addb    soc_at_start      ;Get the original battery SOC
Calc_Soc_2:     cmpb    #99               ;How does it compare to 99%
                bhs     Calc_Soc_3        ;We're too high, save as 99%
                cmpb    #0                ;How are we compared to zero
                bhs     Calc_Soc_4        ;Looking good, save it
                clrb                      ;Clear the B accumulator
                bra     Calc_Soc_4        ;Save it and leave

Calc_Soc_3:     ldab    #99               ;Hard load a 99% value
Calc_Soc_4:     stab    ch_state          ;Our % State of Charge is in the B

;  Set the bit in the special flag byte, indicating the Calculation has been
;  completed for this time period.  Then exit the module.
;
Calc_Soc_5:     bset    Flag1,02h         ;Show the calculation complete
                rts

;  This flag is only cleared when we are on a second not equal to 00 (start of
;  a new minute).  Otherwise we leave the flag set to indicate the first pass
;  was completed and prevent sampling multiple times while the seconds counter
;  is still at 00 due to the micro-controller speed.
;
Calc_Soc_Clr:   bclr    flag1,02h         ;Show the calculation as available
Calc_Soc_Exit:  rts

;****************************************************************************


;****************************************************************************
;  Shift Right, Double Accumulator - 21 June 95 by J.W. Hagen, HTSC
;  Description:  Shifts the contents of the Double Accumulator to the right by
;                the number contained in the Y register.
;  Prog flow:    Linear execution
;  Regs Altered: D,y
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Shift_Right_D:  lsrd                      ;Perform a right shift
                dey                       ;Show as another shift complete
                bne     Shift_Right_D     ;Loop till we're done
Shift_R_Exit:   rts                       ;Back to where we came from

;****************************************************************************


;****************************************************************************
;  Shift Left, Double Accumulator - 21 June 95 by J.W. Hagen, HTSC
;  Description:  Shifts the contents of the Double Accumulator to the left by
;                the number contained in the Y register.
;  Prog flow:    Linear execution
;  Regs Altered: D,X
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Shift_Left_D:   lsld                      ;Perform a left shift
                dey                       ;Show as another shift complete
                bne     Shift_Left_D      ;Loop till we're done
Shift_L_Exit:   rts                       ;Back to where we came from

;****************************************************************************


;****************************************************************************
;  Updt Mode Time 26 Apr 95 by J.W. Hagen, HTSC
;  Description:   Updates the mode elapsed time for the respective mode.
;  Prog flow:     Linear.
;  Regs Altered:  A
;  Called By:     Rti_Irq
;  Global Read:   mone_hour_count, mone_min_count, mone_sec_count
;  Global Write:  None
;  Passed value:  None.
;  Return value:  None.
;
Updt_Mode_Time: bsr     Get_Parm_Add      ;Return parameter table pointer in X
                cmpa    #MODE0            ;Is it Mode 0
                beq     Updt_Mode_Rts     ;Yes, exit
                cmpa    #MODE254          ;Completion or Fault Mode
                bcc     Updt_Mode_Rts     ;Yes, exit

;  Only update timers when in Mode 1,2,3, or 4.
;
                ldaa    mone_hour_count   ;This mode's elapsed time in hours
                staa    5,x               ;Save in the mode parameter table
                ldaa    mone_min_count    ;This mode's elapsed time in minutes
                staa    6,x               ;Save in the mode parameter table
                ldaa    mone_sec_count    ;This mode's elapsed time in seconds
                staa    7,x               ;Save in the mode parameter table

Updt_Mode_Rts:  rts                       ;Return to the calling program

;****************************************************************************


;******************************************************************************
;  Get Parm Address - 26 Apr 95 by J.W. Hagen, HTSC
;  Description:   Determines the mode presently being processed and places the
;                 first address in the parameter table in to the X register.
;                 Use of extra rts commands is for efficiency and reduction of
;                 EEPROM program space requirements.
;  Prog flow:     Linear.
;  Regs Altered:  A, X
;  Called By:     Updt_Mode_Time, Control, Temp_Comp
;  Global Read:   Mode
;  Global Write:  None
;  Passed value:  None.
;  Return value:  A contains the present Mode
;                 X contains first address in respective mode parameter table
;
Get_Parm_Add:   ldaa    mode              ;What mode are we in at this time
                cmpa    #MODE1            ;Is it mode 1
                beq     Get_Parm_Add1     ;Yes, get address for Mode 1 params.
                cmpa    #MODE2            ;Is it mode 2
                beq     Get_Parm_Add2     ;Yes, get address for Mode 2 params.
                cmpa    #MODE3            ;Is it mode 3
                beq     Get_Parm_Add3     ;Yes, get address for Mode 3 params.
                cmpa    #MODE4            ;Is it mode 4
                beq     Get_Parm_Add4     ;Yes, get address for Mode 4 params.
                rts                       ;Not Mode 1,2,3, or 4.

Get_Parm_Add1:  ldx     #mode1_parms      ;Point to Mode 1 parameter table
                rts

Get_Parm_Add2:  ldx     #mode2_parms      ;Point to Mode 2 parameter table
                rts

Get_Parm_Add3:  ldx     #mode3_parms      ;Point to Mode 3 parameter table
                rts

Get_Parm_Add4:  ldx     #mode4_parms      ;Point to Mode 4 parameter table
                rts

;****************************************************************************


;****************************************************************************
;  Select Mode   24 Apr 95 by J.W. Hagen, HTSC
;  Description:  Selects the next Mode, based on the present Mode.
;  Prog flow:    Linear.
;  Regs Altered: A
;  Global Read:  Mode
;  Global Write: Mode
;  Passed value: None.
;  Return value: None.
;
Select_Mode:    ldaa    mode              ;Let's see what Mode we're in
                cmpa    #MODE0            ;Finished Power Up Delay
                beq     Select_1stMode    ;Yes, enter the first mode
                cmpa    #FIRST_MODE       ;Was it the 1st Mode
                beq     Select_2ndMode    ;Yes, enter the second mode
                cmpa    #SECOND_MODE      ;Was it the 2nd Mode
                beq     Select_3rdMode    ;Yes, enter the third mode
                cmpa    #THIRD_MODE       ;Was it the 3rd Mode
                beq     Select_4thMode    ;Yes, enter the fourth mode
                cmpa    #FOURTH_MODE      ;Was it the 4th Mode
                beq     Select_Done       ;Yes, enter the Completion Mode
                ldaa    #MODE255          ;Default is Fault Mode
                bra     Select_Rts        ;Save it and exit

Select_1stMode: ldaa    #FIRST_MODE       ;Get the 1st Mode number
                bra     Select_Rts        ;Save it and exit

Select_2ndMode: ldaa    #SECOND_MODE      ;Get the 2nd Mode number
                bra     Select_Rts        ;Save it and exit

Select_3rdMode: ldaa    #THIRD_MODE       ;Get the 3rd Mode number
                bra     Select_Rts        ;Save it and exit

Select_4thMode: ldaa    #FOURTH_MODE      ;Get the 4th Mode number
                bra     Select_Rts        ;Save it and exit

Select_Done:    ldaa    #MODE254          ;Get the Completion Mode number

Select_Rts:     staa    mode              ;Save as our new mode
                bset    Flag1,010h        ;Set Mode transition flag
                rts                       ;Finished here

;****************************************************************************


;****************************************************************************
;  Read_Ports    Modified 06 Jun 95  JWH
;  Description:  Reads Port A, B, C, & D.  Saves data to port_a_status,
;                port_b_status, port_c_status, and port_d_status.
;  Prog flow:    Linear execution, called by RTI IRQ, no local calls.
;  Regs Altered: a, x
;  Global Read:  None.
;  Global Write: port_a_status, port_b_status, port_c_status, port_d_status
;  Passed value: None.
;  Return value: REGBAS in the X
;
Read_Ports:     ldx     #REGBAS           ;Offset to registers

;  Process Port A
;
                ldaa    PORTA,x           ;All Discretes on PA0-PA7
                coma                      ;Invert Signals 16Feb93 rm
                staa    port_a_status     ;Set status bits

;  Process Port B
;
;               ldaa    PORTB,x           ;All Discretes on PB0-PB7
;               coma                      ;Invert Signals
;               staa    port_b_status     ;Set status bits

;  Process Port C
;
                ldaa    PORTC,x           ;All Discretes on PC0-PC7
                coma                      ;Invert Signals 16Feb93 rm
                staa    port_c_status     ;Set status bits

;  Process Port D
;
;               ldaa    PORTD,x           ;All Discretes on PD0-PD5
;               coma                      ;Invert Signals
;               staa    port_d_status     ;Set status bits

                rts                       ;Back to the calling program

;****************************************************************************


;******************************************************************************
;  Clear_PA7     Modified 12 Sep 95 by JWH
;  Description:  Clear the Discrete Output if we're in Completion or Fault
;                Mode.  This will then put to sleep any external connections
;                made to the discrete output.  The discrete will turn-off once
;                in Completion or Fault Mode at least Delay_PA7 seconds.
;  Prog flow:    Linear.
;  Regs Altered: A,X
;  Global Read:  PortA, Mode
;  Global Write: PortA, Bit 7
;  Passed value: None.
;  Return value: None.
;
DELAY_PA7:      equ     10                ;Set for a 10 second delay
;
Clear_PA7:      ldaa    mode              ;Let's see what Mode we're in
                cmpa    #MODE254          ;At least reached Completion Mode
                bcs     Clear_PA7_Rts     ;No, don't clear the D.O. yet

                ldaa    mone_sec_count    ;How many seconds have elapsed
                cmpa    #DELAY_PA7        ;See how we're doing with the delay
                blt     Clear_PA7_Rts     ;Not ready yet, press on

                ldx     #REGBAS           ;Offset to Registers
                bclr    PORTA,x,080h      ;Send it to the port

Clear_PA7_Rts:  rts                       ;Our work is done!

;****************************************************************************


;****************************************************************************
;  putchar       modified 23Feb93 reset the watchdog
;  Description:  Sends an 8 bit ascii char to the SCI serial port.
;  Prog flow:    Linear, waits for TxDataRegEmpty SCI flag clear.
;  Regs Altered: None.
;  Global Read:  None.
;  Global Write: None.
;  Passed value: Data to send passed in register a.
;  Return value: None.
;
Putchar:        pshx                      ;store x
                ldx     #REGBAS           ;register offset
Putchar1:       brclr   SCSR,x,80h,putchar1  ;spin on tdre
                staa    SCDR,x            ;output char
                pulx                      ;restore x
                rts                       ;We're done here
 
;****************************************************************************


;****************************************************************************
;  reset_watchdog
;  Description:  resets watchdog timer, which times out every 1 sec.
;  Prog flow:    Linear execution
;  Regs Altered: COPRST,a
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Reset_Watchdog: psha                      ;Push the A on the stack
                pshx                      ;Push the X on the stack
                ldx     #REGBAS           ;Load the register base address
                ldaa    #055h             ;reset COP watchdog, 1st step of 2
                staa    COPRST,x          ;timer
                ldaa    #0aah             ;reset COP watchdog, 2nd step of 2
                staa    COPRST,x          ;
                pulx                      ;Restore the X from the stack
                pula                      ;Restore the A from the stack
                rts                       ;We're done

;****************************************************************************


;****************************************************************************
;  Ram_Out - 01 Aug 94 by J.W. Hagen, HTSC
;  Description:  This routine sends all of the Ram data out the RS232 port.
;  Prog flow:    Linear.
;  Regs Altered: Y,X,A
;  Global Read:  Ram Data
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Ram_Out:        jsr     Reset_Watchdog    ;reset COP watchdog first

;  Send the Ram Data
;
                ldy     #0                ;Starting Address of Ram to Dump
                ldx     #0                ;Clear the X register

Ram_Out1:       ldaa    0,y               ;Load the Ram data, starting at 00h
                jsr     Putchar           ;Send this byte
                iny                       ;Increment the address offset
                inx                       ;Increment our byte counter
                cpx     #256              ;Have we sent 256 bytes yet
                blt     Ram_out1          ;No, loop till we're done
Ram_Rts:        rts                       ;We're done, bail out


;****************************************************************************
;  Critical integration parameters control routine accesses.  Do NOT change the
;  order of this data.  It must coincide with the dianostic program.
;
Ram_Setup:      dfb     250               ;1st Sync Byte
                dfb     243               ;2nd Sync Byte
                dfb     32                ;3rd Sync Byte

                dfb     "Ev14.043"        ;Name of this program (xxxxxxxx.ASM)
                                          ;MUST always be 8 bytes.
                                          ;NEVER change this information!

                dfb     "011196"          ;Program Revision Date (mm/dd/yy)
                                          ;MUST always be 6 bytes followed by a
                                          ;comma and a zero.

                dwm     VOLTS_MULTIPLY    ;Multiplier used for Voltage
                dwm     VOLTS_SCALE       ;Voltage Scaled * Voltage_Multiply

                dwm     AMPS_MULTIPLY     ;Multiplier used for Current
                dwm     AMPS_SCALE        ;Current Scaled * Current_Multiply

                dfb     TYPE_BATTERY      ;Code for Type of Battery
                dfb     STRINGS           ;Number of Battery Strings
                dfb     BATTERIES         ;Number of Batteries
                dfb     CELLS             ;Number of Cells per battery
                dfb     AMP_HOURS         ;Battery Ampere Rating
                dwm     MAX_AMP_MINS      ;Max Amp Mins allowed to be returned
                dfb     VMAX              ;Maximum Voltage
                dfb     VMIN              ;Minimum Voltage

                dfb     SAMPLES_HOUR      ;Number of Current samples per hour

                dfb     TEMPMAX           ;Maximum Temperature
                dfb     TEMPMIN           ;Minimum Temperature for Charging
 
Mode1_Cc:       dfb     T1_HOURS          ;Mode 1 maximum hours allowed
                dfb     VTRIP1            ;Voltage trip for mode 1
                dfb     I1                ;Set current for mode 1
                dfb     I_COR             ;Scaled current gain (Kcc)
                dfb     T1_MIN            ;Mode 1 maximum mins allowed
                                          ; (hours must be reached first)
                dfb     0                 ;Hours elapsed in this Mode
                dfb     0                 ;Minutes elapsed in this mode
                dfb     0                 ;Seconds elapsed in this Mode
                dfb     0                 ;Applied Amp Minutes MSByte
                dfb     0                 ;Applied Amp Minutes LSByte
 
Mode2_Cv:       dfb     T2_HOURS          ;Mode 2 maximum hours allowed
                dfb     ITRIP1            ;Current trip for mode 2
                dfb     V1                ;Set voltage for mode 2
                dfb     V_COR             ;Scaled voltage gain (Kcv)
                dfb     T2_MIN            ;Mode 2 maximum mins allowed
                                          ; (hours must be reached first)
                dfb     0                 ;Hours elapsed in this Mode
                dfb     0                 ;Minutes elapsed in this mode
                dfb     0                 ;Seconds elapsed in this Mode
                dfb     0                 ;Applied Amp Minutes MSByte
                dfb     0                 ;Applied Amp Minutes LSByte
 
Mode3_Cc:       dfb     T3_HOURS          ;Mode 3 maximum hours allowed
                dfb     VTRIP2            ;Voltage trip for mode 3
                dfb     I2                ;Set current for mode 3
                dfb     I_COR             ;Scaled current gain (Kcc)
                dfb     T3_MIN            ;Mode 3 maximum mins allowed
                                          ; (hours must be reached first)
                dfb     0                 ;Hours elapsed in this Mode
                dfb     0                 ;Minutes elapsed in this mode
                dfb     0                 ;Seconds elapsed in this Mode
                dfb     0                 ;Applied Amp Minutes MSByte
                dfb     0                 ;Applied Amp Minutes LSByte

Mode4_Cv:       dfb     T4_HOURS          ;Mode 4 maximum hours allowed
                dfb     ITRIP2            ;Current trip for mode 4
                dfb     V2                ;Set voltage for mode 4
                dfb     V_COR             ;Scaled voltage gain (Kcv)
                dfb     T4_MIN            ;Mode 4 maximum mins allowed
                                          ; (hours must be reached first)
                dfb     0                 ;Hours elapsed in this Mode
                dfb     0                 ;Minutes elapsed in this mode
                dfb     0                 ;Seconds elapsed in this Mode
                dfb     0                 ;Applied Amp Minutes MSByte
                dfb     0                 ;Applied Amp Minutes LSByte
;
;End of mode parameter table

                dfb     FIRST_SECOND      ;1st & 2nd Mode to be processed
                dfb     THIRD_FOURTH      ;3rd & 4th Mode to be processed

                dfb     AMPHR_TRIP_2      ;% of Mode1 AmpHrs to apply to Mode2

                dfb     TMAX              ;Maximum Charging Time

                dfb     Temp_Max_Volts    ;Maximum Temp Compensated Voltage
                dfb     Temp_Min_Volts    ;Minimum Temp Compensated Voltage

                dfb     AmpsPerLevel      ;# of Amps per level, Mode 5
                dfb     LEVELS            ;Total Levels for Mode 3
                dfb     LEVELS            ;Levels Remaining
                dfb     AMPHR_TRIP_3      ;% of Mode1&2 AmpHrs to apply Mode3

Ram_Last_Byte:  equ     $                 ;End of data being sent to Ram

;****************************************************************************


;****************************************************************************
;   Interrupt vector orgs:
                org     0ffdeh
                dwm     tof_irq           ;Timer Overflow
 
                org     0ffe6h
                dwm     oc2_irq           ;Output Compare 2 IRQ
 
                org     0ffe8h
                dwm     oc1_irq           ;Output Compare 1 IRQ
 
                org     0fff0h
                dwm     rti_irq           ;Real Time Interrupt
 
                org     0fff2h
                dwm     dlcs_irq          ;DLCS Interrupt (On *IRQ pin)
 
                org     0fffah            ;COP causes cold start
                dwm     RESET
 
                org     0fffeh            ;Reset Vector
                dwm     RESET
